package com.iflytek.cloud;

import android.os.Bundle;



/**
 * <h3>合成监听器</h3>
 * <p>通过实现此接口，获取当前合成的状态和结果</p>
 * <p>识别监听器，请参考{@link RecognizerListener}</p>
 * <p>语音语义监听器，请参考{@link SpeechUnderstanderListener}</p>
 * <p>文本语义监听器，请参考{@link TextUnderstanderListener}</p>
 *
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see SpeechSynthesizer#startSpeaking(String, SynthesizerListener)
 * @see RecognizerListener
 * @see SpeechUnderstanderListener
 * @see TextUnderstanderListener
 */
public abstract interface SynthesizerListener {
	
	/**
	 * <h4>开始播放</h4>
	 * <p>
	 * 		SDK回调此函数，通知应用层，将要进行播放。
	 * </p>
	 * <p>
	 * 		在第一次回调了{@link #onBufferProgress(int, int, int, String)}后，便会回调此
	 * 函数，告知应用层，将要进行播放。此函数在一次会话中，只会被调用一次，当出现
	 * 错误时，可能不会回调此函数。
	 * </p>
	 * @see #onBufferProgress(int, int, int, String)
	 * @see #onSpeakProgress(int, int, int)
	 */
	void onSpeakBegin();
	
	/**
	 * <h4>缓冲进度</h4>
	 * <p>
	 * 		SDK回调此函数，通知应用层，当前合成音频的缓冲进度。
	 * </p>
	 * <p>
	 * 		除非出现错误，否则，在合成过程，第一个回调的应当是此函数。此函数在
	 * 一次会话中，可能会被调用多次，直到参数3的值为( textLen-1 )为止，textLen
	 * 即为在{@link SpeechSynthesizer#startSpeaking(String, SynthesizerListener)}
	 * 传入的文本长度。当出现错误时，可能不会回调此函数。缓冲进度的文本结束位
	 * 置一般为文本中句子停顿位置，如逗号，句号等。
	 * </p>
	 * 
	 * @param progress 百分比，当前已合成文本占当前会话全部文本的百分比。
	 * @param beginPos 文本开始位置，从0开始计数。
	 * @param endPos 文本结束位置，最大值为（textLen-1）。
	 * @param info 信息，暂不支持。
	 * 
	 * @see #onSpeakProgress(int, int, int)
	 */
	void onBufferProgress(int progress,int beginPos,int endPos, String info);
	
	/**
	 * <h4>暂停播放</h4>
	 * <p>
	 * 		SDK回调此接口，通知应用，将暂停播放。
	 * </p>
	 * <p>
	 * 		仅在当前缓冲音频已播完，下一段音频未到时，SDK回调此函数，告知
	 * 应用层，将暂停播放。而当应用主动调用
	 * {@link SpeechSynthesizer#pauseSpeaking()}暂停播放时，不会回调此函数。
	 * </p>
	 * @see #onSpeakResumed()
	 */
	void onSpeakPaused();
	
	/**
	 * <h4>恢复播放</h4>
	 * <p>
	 * 		SDK回调此接口，通知应用，将恢复播放。
	 * </p>
	 * <p>
	 * 		仅在当前暂停由于音频缓存未到而引起的暂停，再恢复播放时，SDK回调此
	 * 函数，告知应用层，将恢复播放。而当应用主动调用
	 * {@link SpeechSynthesizer#pauseSpeaking()}暂停播放后，再调用
	 * {@link SpeechSynthesizer#resumeSpeaking()}恢复播放时，不会回调此函数。
	 * </p>
	 * @see #onSpeakPaused()
	 */
	void onSpeakResumed();
	
	/**
	 * <h4>播放进度</h4>
	 * <p>
	 * 		SDK回调此接口，通知应用，当前的播放进度。
	 * </p>
	 * <p>
	 * 		在回调了{@link #onSpeakBegin()}后，便会回调此函数，告知应用层，当前的播
	 * 放进度。此函数在一次会话中，可能会被回调多次，且被调用次数与
	 * {@link #onBufferProgress(int, int, int, String)}的被用次数一样。且，在同一次被调
	 * 用时，此函数的回调中的文本开始位置和结束位置，与对应那次被调用的
	 * {@link #onBufferProgress(int, int, int, String)}的值一致。然而，除了第一次外，播
	 * 放进度，总是比缓冲进度慢，即可能全部文本已缓冲完成，播放进度还没开始下一次
	 * 的回调。当出现错误时，可能不会回调此函数。
	 * </p>
	 * 
	 * @param progress 百分比，当前待播放音频，占已合成音频数据长度的百分比。
	 * @param beginPos 文本开始位置。
	 * @param endPos 文本结束位置。
	 * 
	 * @see #onBufferProgress(int, int, int, String)
	 */
	void onSpeakProgress(int progress,int beginPos,int endPos);
	
	/**
	 * <h4>结束</h4>
	 * <p>
	 * 		SDK回调此接口，通知应用，将结束会话。
	 * </p>
	 * <p>
	 * 		在音频播放完成，或会话出现错误时，将回调此函数。若应用主动调用
	 * {@link SpeechSynthesizer#stopSpeaking()}停止会话，则不会回调此函数。
	 * </p>
	 * 
	 * @param error 错误信息，若为null，则没有出现错误。
	 * 
	 * @see SpeechSynthesizer#stopSpeaking()
	 */
	void onCompleted(SpeechError error);
	
    /**
     * <h4>合成会话事件</h4>
     * <p>
     * 		扩展用接口，由具体业务进行约定。
     * </p>
     * 
     * @param eventType 消息类型
     * @param arg1 参数1
     * @param arg2 参数2
     * @param obj 扩展参数
     * 
     * @see SpeechEvent
     */
    void onEvent(int eventType, int arg1,int arg2, Bundle obj);
}
