package com.iflytek.cloud;

import android.os.Bundle;

/**
 * <h4>转写回调接口</h4>
 * <script>gShowExtraTranscripter();</script>
 * <p>
 * 		通过实现此接口，获取转写的状态和结果。
 * </p>
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see com.iflytek.cloud.SpeechTranscripter
 * @see com.iflytek.cloud.TranscripterResult
 * @see com.iflytek.cloud.SpeechEvent
 */
public abstract interface TranscripterListener {
	/**
	 * <h4>音量变化</h4>
	 * <p>
	 *     当开始录音，到停止录音（停止写入音频流）或SDK返回最后一个结果自动结束识别为止，
	 *  SDK检测到音频数据（正在录音或写入音频流）的音量变化时，会多次通过此函数回调，告
	 *  知应用层当前的音量值。应用层可通过此函数传入的值变化，改变自定义UI的画面等。
	 * </p> 
	 * @param volume 当前音量值，范围[0-30]
	 * @param data 录音数据，格式请参考{@link SpeechConstant#SAMPLE_RATE}
	 * 		, {@link SpeechConstant#AUDIO_FORMAT}。
	 * 
	 * @see SpeechTranscripter#startTranscripting
	 * @see #onBeginOfSpeech
	 * @see #onEndOfSpeech
	 * @see SpeechTranscripter#stopTranscripting
	 */
	void onVolumeChanged(int volume,byte[] data);
	
	/**
	 * <h4>开始说话</h4>
	 * <p>
	 * 		在录音模式({@link SpeechConstant#AUDIO_SOURCE 音频源}参数设为 > -1时 )下，
	 * 调用{@link SpeechRecognizer#startListening 开始录音函数}后，会自动开启系统的录音
	 * 机，并在录音机开启后，会回调此函数（这中间的过程应该在几毫秒内，可以忽略，除非系
	 * 统响应很慢）。
	 * </p>
	 * <p>
	 * 		应用层可通过此函数回调，告知用户，当前可开始说话。若应用层使用的是声音提示，
	 * 则应该在调用{@link SpeechTranscripter#startTranscripting 开始录音函数}前，提放提示音，
	 * 并在提示音非静音数据播放结束时，调用
	 * {@link SpeechTranscripter#startTranscripting 开始录音函数}，开始录音。若太早调用，则
	 * 可能会把提示音录进要识别的音频中，若太晚，则可能会漏掉部分用户说话的音频。
	 * </p>
	 * @see SpeechTranscripter#startTranscripting
	 * @see #onEndOfSpeech
	 */
	void onBeginOfSpeech();

	/**
	 * <h4>结束说话</h4>
	 * <p>
	 * 		主动调用{@link SpeechTranscripter#stopTranscripting}停止录音，或录音超时，
	 * 回调此函数（在识别出错时，可能不会回调此函数）。在此函数回调后，当前会话可能
	 * 并没有结束，转写结果可能还要等待一定的时间才会返回；音频可能还未完成上传到服
	 * 务器。参考事件{@link SpeechEvent#EVENT_IST_RESULT_TIME}，
	 * {@link SpeechEvent#EVENT_IST_UPLOAD_BYTES}。
	 * </p>
	 * <p>
	 * 		应用层可以通过此函数回调，告知用户，当次录音已结束，正在等待结果（若结果
	 * 未返回）等。
	 * </p>
	 * 
	 * @see #onBeginOfSpeech
	 * @see #onResult
	 * @see #onError
	 * @see SpeechTranscripter#writeAudio
	 * @see SpeechTranscripter#stopTranscripting
	 */
	void onEndOfSpeech();

	/**
	 * <h4>返回结果</h4>
	 * <p>
	 * 		返回的结果可能为null，请增加判断处理。
	 * </p>
	 * <p>
	 * 		一次识别会话的结果可能会多次返回（即多次回调此函数），通过参数2,判断是否是最后
	 * 一个结果，true时为最后一个结果，否则不是。
	 * 当出现错误，或应用层调用{@link SpeechTranscripter#cancel}取消当次识别时，在当次识
	 * 别会话过程可能不会回调此函数。
	 * </p>
	 * <p>
	 * 		识别采用边录边上传的分次上传音频数据方式，可能在结束录音前，就有结果返回。
	 * </p>
	 * 
	 * @param result 转写结果，默认是未解析的Json结果，请参考{@link TranscripterResult}
	 * @param islast 是否最后一次的结果
	 * 
	 * @see SpeechEvent#EVENT_IST_RESULT_TIME
	 * @see TranscripterResult
	 */
	void onResult(TranscripterResult result,boolean islast);

	/**
	 * <h4>错误回调</h4>
	 * <p>
	 * 		当此函数回调时，说明当次会话出现错误，会话自动结束，录音也会停止。应在再次调用
	 * {@link SpeechTranscripter#startTranscripting}开启新的识别会话前，
	 * 停止调用{@link SpeechTranscripter#writeAudio}写入音频(当音频源设置为音频流时
	 * （{@link SpeechConstant#AUDIO_SOURCE}为-1时)。
	 * </p>
	 * 
	 * @param error 错误类型，获取错误码信息请查看{@link SpeechError} 详细错误码请看{@link ErrorCode}
	 * 
	 * @see SpeechTranscripter#startTranscripting
	 * @see SpeechTranscripter#writeAudio
	 * @see SpeechError
	 * @see ErrorCode
	 */
	void onError(SpeechError error);
	
    /**
     * <h4>事件</h4>
     * <p>
     * 		转写支持的事件，包括以下几个：
     * <br>{@link SpeechEvent#EVENT_IST_AUDIO_FILE}：录音文件；
     * <br>{@link SpeechEvent#EVENT_IST_CACHE_LEFT}：缓存剩余；
     * <br>{@link SpeechEvent#EVENT_IST_RESULT_TIME}：结果等待时间；
     * <br>{@link SpeechEvent#EVENT_IST_SYNC_ID}：同步ID；
     * <br>{@link SpeechEvent#EVENT_IST_UPLOAD_BYTES}：已上传字节数；
     * <br>{@link SpeechEvent#EVENT_SESSION_BEGIN}：会话开始；
     * <br>{@link SpeechEvent#EVENT_SESSION_END}：会话结束；
     * <br>{@link SpeechEvent#EVENT_SESSION_ID}：会话ID；
     * </p>
     * @param eventType 消息类型
     * @param arg1 参数1
     * @param arg2 参数2
     * @param obj 消息内容
     * 
     * @see SpeechEvent
     */
	void onEvent(int eventType, int arg1,int arg2, Bundle obj);
}
