/*
 * 文件名：VoiceWakeuper.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：唤醒对象，包括开始唤醒、取消唤醒、停止唤醒、查询模型及下载模型等接口；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud;

import android.content.Context;

import com.iflytek.cloud.msc.ivw.aimic.AIMic;
import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.speech.impl.VoiceWakeuperImpl;
import com.iflytek.cloud.util.FileDownloadListener;
import com.iflytek.cloud.util.ResourceUtil;

/**
 * <h3>语音唤醒类</h3>
 * <p>
 * 		唤醒包括：单唤醒，多唤醒，特定人唤醒，唤醒识别。见{@link SpeechConstant#IVW_SST}。
 * 其中，特定人唤醒暂不支持。
 * </p>
 * <p>
 *  	1.	单唤醒：说出唤醒词，返回唤醒消息，其它情况无消息返回。<br>
 * </p>
 * <p>
 * 		2.	多唤醒：用户说出预定的几个唤醒词中任意一个，给用户反馈唤醒，并区分由什么词唤醒。
 * </p>
 * <p>
 * 		在唤醒时，单唤醒、多唤醒使用基本一致，只需要通过{@link #setParameter}
 * 设置资源路径 {@link SpeechConstant#IVW_RES_PATH}等参数后，
 * {@link #startListening 开始唤醒}即可。
 * </p>
 * <p>
 * 		3.	唤醒识别：说出预定义的唤醒词接着识别词，引擎给出唤醒识别结果。
 * 	例如，“打电话给XXX”；其中“打电话给”是固定的，用唤醒引擎检测，是唤醒词；
 * “XXX”是人名，不固定，用识别引擎检测；当用户说出“打电话给XXX”时，引擎返
 * 回唤醒识别结果。唤醒识别在{@link #startListening 开始识别}前，需要设置
 * {@link SpeechConstant#CLOUD_GRAMMAR 云端语法}或
 * {@link SpeechConstant#LOCAL_GRAMMAR 本地语法}（视使用本地还是云端识别而定）。
 * </p>
 * <p>
 * 		由于唤醒开始后，会一直占用MIC（{@link SpeechConstant#AUDIO_SOURCE 音频源}为MIC时），
 * 开发安卓手机的唤醒应用时，应该通过实现{@link android.content.BroadcastReceiver#onReceive},
 * 监听系统广播{@link android.content.Intent#ACTION_SCREEN_ON}，
 * 在屏幕点亮时，通过{@link #cancel}取消会话；监听系统广播
 * {@link android.content.Intent#ACTION_SCREEN_OFF}，在屏幕锁住时，通过{@link #startListening}
 * 开启唤醒会话。这样在屏幕点亮时，使用其他应用就不会出现由于唤醒在占用MIC，而无法使用MIC的问题。
 * </p>
 * <p>
 * 		另外，由于唤醒一般在幕屏息灭时起作用，就要求集成唤醒功能时，应该作为服务{@link android.app.Service}，
 * 运行在后台，避免由于系统清理应用，而导致唤醒不起作用的问题。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createWakeuper}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getWakeuper}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createWakeuper}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechUtility#createUtility
 */
public class VoiceWakeuper extends SpeechInterface{
	
	private static VoiceWakeuper sInstance = null;
	
	private VoiceWakeuperImpl mWaker = null;
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getWakeuper}获取已创建的单例对象。
	 * </p>
	 * 
	 * @return 唤醒对象
	 * @see #destroy
	 * @see #getWakeuper
	 */
	public synchronized static VoiceWakeuper createWakeuper(Context context,InitListener listener) {
		synchronized( sSync ){
			if( sInstance==null && null!=SpeechUtility.getUtility() ){
				sInstance = new VoiceWakeuper(context,listener);
			}
		}//end of sync
		
		return sInstance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createWakeuper}创建单例对象。
	 * </p>
	 * 
	 * @return 唤醒对象
	 * 
	 * @see #createWakeuper
	 * @see #destroy
	 */
	public static VoiceWakeuper getWakeuper() {
		return sInstance;
	}

	/**
	 * 初始化唤醒对象。<br>
	 * @param context 上下文
	 */
	private VoiceWakeuper(Context context,InitListener listener) {
		super();
		mWaker = new VoiceWakeuperImpl(context);
	}

	/**
	 * <h4>查询唤醒词资源</h4>
	 * <p>
	 * 		调用此函数，查询唤醒词优化资源，能够查询的前提是应用开启唤醒闭环优化，
	 * 并且云端成功训练出唤醒词优化资源，调用此函数发送查询请求，若有优化的模型
	 * 资源，会返回开发者相应的资源下载链接，如果开发者未开启唤醒闭环优化功能，
	 * 此函数会通过回调监听返回相应错误码
	 * </p>
	 * <p>
	 * 		调用此函数仅仅模型查询，下载模型资源请参考{@link #downloadResource}
	 * 注：该接口仅加强型支持，基础性暂无优化功能
	 * </p>
	 * 
	 * @param resourcePath
	 * 			唤醒资源路径
	 * @param listener
	 * 			查询回调监听
	 * @return
	 * 			是否启动查询成功，{@link ErrorCode#SUCCESS}表示成功，其他值表示有错误，
	 * 具体错误码，请参考{@link ErrorCode}。
	 * 
	 * @see #downloadResource
	 */
	public int queryResource(String resourcePath, RequestListener listener) {
		return mWaker.queryResource(resourcePath, true, listener);
	}
	
	/**
	 * <h4>下载唤醒词资源</h4>
	 * <p>
	 * 		调用此函数，通过参数传入下载url和文件保存路径，可下载资源至相应路径目录下。
	 * 并通过回调监听返回当前资源的下载状态
	 * </p>
	 * <p>
	 * 		调用此函数仅支持下载，查询模型资源请参考{@link #queryResource}
	 * 注：该接口仅加强型支持，基础性暂无优化功能
	 * </p>
	 * 
	 * @param downloadUrl
	 * 			资源下载链接
	 * @param filePath
	 * 			下载文件保存路径
	 * @param fileMd5
	 * 			资源md5值，用于检验
	 * @param listener
	 * 			文件下载回调监听	
	 * @return
	 * 			是否启动下载成功，{@link ErrorCode#SUCCESS}表示成功，其他值表示有错误，
	 * 具体错误码，请参考{@link ErrorCode}。
	 * 
	 * @see #queryResource
	 * @deprecated 该方法未被调用过，已废弃使用
	 */
	public int downloadResource(String downloadUrl, String filePath,
			String fileMd5, FileDownloadListener listener) {
		return mWaker.downloadResource(downloadUrl, filePath, fileMd5, true, listener);
	}

	
	/**
	 * <h4>开始录音</h4>
	 * <p>
	 * 		调用此函数，开始唤醒和注册会话。
	 * </p>
	 * <p>
	 * 		目前SDK不支持多线程，所以在调用本函数开始一次会话后，直到结束前(结果返回
	 * 完毕，或出现错误)，不能再调用本函数开始新的会话（否则当前会话将自动结束）。
	 * 一次会话，即从会话开始，到结束为止。
	 * </p>
	 * <p>
	 * 		在调用本函数开始会话前，通过{@link #setParameter}设置相应的参数。包括
	 * {@link SpeechConstant#IVW_RES_PATH 唤醒资源路径}，
	 * {@link SpeechConstant#IVW_SST 唤醒业务类型}等，参考{@link #setParameter}。
	 * </p>
	 * <p>
	 * 		调用本函数开始会话后，通过SDK的录音机录取用户通过麦克风读入的音频（当
	 * {@link SpeechConstant#AUDIO_SOURCE} 值>=0时），或由应用层调用
	 * {@link #writeAudio}写入音频流，获取用于唤醒或注册的音频。在完成音频
	 * 录入（包括麦克风或写音频流方式）后，通过调用{@link #stopListening}告知SDK已完
	 * 成音频录入。
	 * </p>
	 * <p>
	 * 		在开始会话后，如果通过SDK的录音机录取用户通过麦克风读入的音频（当
	 * {@link SpeechConstant#AUDIO_SOURCE} 值>=0时），直到用户读到唤醒词而
	 * {@link WakeuperListener#onResult 唤醒成功}(非{@link SpeechConstant#KEEP_ALIVE 持续唤醒}时)，
	 * 或取消会话，或出现出错{@link WakeuperListener#onError}为止，录音将一直持续。
	 * </p>
	 * <p>
	 * 		可通过{@link #cancel}取消当前的会话。
	 * </p>
	 * <p>
	 * 		唤醒默认启动不抢占音频焦点{@link SpeechConstant#KEY_REQUEST_FOCUS},
	 * 开发者如需请通过{@link #setParameter}设置为抢占焦点
	 * </p>
	 * 
	 * @see #createWakeuper
	 * @see #getWakeuper
	 * @see #stopListening
	 * @see #setParameter
	 * @see #cancel
	 */
	public int startListening(WakeuperListener listener)
	{
		mWaker.setParameter(SpeechConstant.PARAMS, null);
		mWaker.setParameter(mSessionParams);
		return mWaker.startListening(listener);
	}
	
	
	/**
	 * <h4>写入录音数据</h4>
	 * <p>
	 * 		通过调用此函数，把音频数据传给SDK。
	 * </p>
	 * <p>
	 * 		仅在{@link SpeechConstant#AUDIO_SOURCE}值为-1时，需要通过调用本函数，把
	 * 音频数据传给SDK。在写入音频数据前，应先通过{@link #startListening}
	 * 开启会话。在结束录音数据写入后，调用{@link #startListening}告知应用层，当前会话
	 * 音频数据已全部写入。
	 * </p>
	 * <p>
	 * 		非{@link SpeechConstant#KEEP_ALIVE 持续唤醒}下，当SDK回调
	 * {@link WakeuperListener#onResult}返回完结果，或任意模式下，回调
	 * {@link WakeuperListener#onError}返回错误时，会话结束，应该停止当次会话的音频写入。
	 * </p>
	 * <p>
	 * 		支持的音频格式，请参考{@link #setParameter(String, String)}的参数说明。
	 * </p>
	 * 
	 * @param buffer
	 *            要写入的录音数据缓存
	 * @param offset
	 *            实际音频在传入缓存的起始点
	 * @param length
	 *            音频数据长度
	 * @return 
	 * 			  是否成功写入数据，{@link ErrorCode#SUCCESS}表示成功，其他值表示有错误，
	 * 具体错误码，请参考{@link ErrorCode}。
	 * 
	 * @see #startListening
	 * @see #stopListening
	 * @see WakeuperListener
	 */
	public int writeAudio(byte[] buffer,int offset,int length)
	{
		if(null != mWaker && mWaker.isListening()){
			return mWaker.writeAudio(buffer, offset, length);
		}else {
			DebugLog.LogE("VoiceWakeup writeAudio failed, is not running");
			return ErrorCode.ERROR_ENGINE_CALL_FAIL;
		}
	}
	
	/**
	 * <h4>停止录音</h4>
	 * <p>
	 * 		调用本函数告知SDK，当前会话音频已全部录入。
	 * </p>
	 * <p>
	 * 		在识别唤醒时，调用本函数后，已录入的音频还在继续识别，结果不会马上就返回，当前会
	 * 话还在继续，直到结果返回完毕，或出现错误。
	 * </p>
	 * <p>
	 * 		在单次或持续唤醒时，本函数与{@link #cancel}相当，即调用本函数后，会话自动结束，不
	 * 会再有结果返回。
	 * </p>
	 * <p>
	 * 		要取消会话，请参考{@link #cancel}函数。
	 * </p>
	 * 
	 * @see #startListening
	 * @see #writeAudio
	 * @see WakeuperListener
	 * @see #cancel
	 */
	public void stopListening()
	{
		mWaker.stopListening();
	}
	
	/**
	 * <h4>是否在会话中</h4>
	 * <p>
	 * 		通过此函数，获取当前SDK是否正在进行会话。应用层可通过此函数，查询能否
	 * 开始一路新的会话等。
	 * </p>
	 * <p>
	 * 		调用了{@link #stopListening}停止录音后，如果会话未出现错误或返回最后的结果，
	 * 当前状态依然处于会话中，即本函数会返回true。如果调用{@link #cancel}取消了会话，
	 * 则当前状态处于不在会话中。
	 * </p>
	 * @return 会话状态，true：正在会话中；false：不在会话中。
	 * 
	 * @see #startListening
	 * @see #stopListening
	 * @see #cancel
	 */
	public boolean isListening()
	{
		return mWaker.isListening();
	}

//	/**
//	 * <h4>更新唤醒词</h4>
//	 * <p>
//	 * 		<b>注意：此函数会阻塞调用的线程，建议别在UI线程调用。</b>
//	 * </p>
//	 * <p>
//	 * 		更新唤醒词，用于合并(merge)已有的两个唤醒资源，或删除(delete)资源中的某个唤醒词
//	 * （目前暂不支持删除功能）。
//	 * </p>
//	 * <p>
//	 * 		更新唤醒词参数仅在此函数的参数2设置起作用，通过{@link #setParameter}
//	 * 设置的参数，对此函数不起作用。更新唤醒词参数，包括：
//	 * <br>	{@link SpeechConstant#IVW_WORD_PATH}：合并资源目标路径。
//	 * <br>	{@link SpeechConstant#IVW_RES_PATH}：待合并资源路径。因合并资源一定是包含
//	 * 两个以上的资源，两个资源之间，以英文的分号";"分隔，并通过
//	 * {@link com.iflytek.cloud.util.ResourceUtil#generateResourcePath}生成标准路径，
//	 * 如 generateResourcePath( context, 
//	 * {@link com.iflytek.cloud.util.ResourceUtil.RESOURCE_TYPE#path}, 
//	 * "/sdcard/res_01.jet;/sdcard/res_02.jet" )；。
//	 * </p>
//	 * @param action 唤醒词操作：merge、delete。
//	 * @param params 操作参数，见上文。
//	 * @return 错误码，若值为{@link ErrorCode#SUCCESS}成功，否则失败，详细错误码，请
//	 * 		参考{@link ErrorCode}类。
//	 * 
//	 * @see #startListening
//	 * @see com.iflytek.cloud.util.ResourceUtil#generateResourcePath
//	 */
//	public int updateWords(String action, String params)
//	{
//		return waker.updateWords(action, params);
//	}
	
	/**
	 * <h4>取消会话</h4>
	 * <p>
	 * 		通过此函数取消当前的会话。
	 * </p>
	 * <p>
	 * 		在会话被取消后，当前唤醒会话结束，未返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #startListening
	 * @see #stopListening
	 */
	public void cancel() {
		mWaker.cancel(false);
	}
	
	/**
	 * <h4>参数设置</h4>
	 * <p>
	 * 		可设置的参数有：
	 * <br>{@link SpeechConstant#IVW_SST}：唤醒类务类型，必须；
	 * <br>{@link SpeechConstant#IVW_THRESHOLD}：唤醒门限值；
	 * <br>{@link SpeechConstant#KEEP_ALIVE}：持续唤醒；
	 * <br>{@link SpeechConstant#IVW_RES_PATH}：唤醒资源路径，必须；
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型；（在唤醒里，引擎类型
	 * 		仅作用于唤醒识别时的识别部分，而唤醒部分始终在离线模式。）
	 * <br>{@link SpeechConstant#IVW_NET_MODE}：唤醒闭环优化模式；<!--（使用闭环模式2
	 * 		必须在setParameter时传入资源路径，否则闭环功能无法执行）-->
	 * <br>{@link SpeechConstant#SAMPLE_RATE}：唤醒音频采样率；
	 * <br>{@link SpeechConstant#CLOUD_GRAMMAR }：云端语法ID；
	 * <br>{@link SpeechConstant#LOCAL_GRAMMAR }：本地语法ID；
	 * <br>{@link SpeechConstant#AUDIO_SOURCE }：音频源，使用 AIMIC 时必须；
	 * <br>{@link SpeechConstant#BLUETOOTH}: 蓝牙录音
	 * <br>{@link SpeechConstant#RESULT_TYPE }：识别结果类型；
	 * <br>{@link SpeechConstant#KEY_REQUEST_FOCUS }：音频焦点；
	 * <br>{@link SpeechConstant#IVW_CHANNEL_NUM }: 唤醒音频路数，使用 AIMIC 时必须；
	 * <br>{@link SpeechConstant#IVW_AUDIO_PATH }: 唤醒音频保存路径；
	 * <br>{@link SpeechConstant#IVW_ALSA_CARD }: ALSA 录音卡号；
	 * <br>{@link SpeechConstant#IVW_ALSA_RATE }: ALSA 录音采样率；
	 * 
	 * <br>	目前唤醒仅支持未压缩的16位，单声道，采样率为16000，字节顺序为
	 *  Little-Endian的Windows PCM音频。另参考 {@link SpeechConstant#IVW_ALSA_RATE } 说明。
	 *  <br>唤醒识别若使用本地识别时，需要有相应的资源文件才支持对应的采样率的音频识别。
	 * </p>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return true 设置成功，否则设置失败
	 * 
	 * @see #startListening
	 * @see #getParameter
	 */
	@Override
	public boolean setParameter(String key, String value) {
		// 若是AIMIC即时参数，则不经过SSB的参数，而立即设置到AIMIC中
		if( null!=key && key.startsWith(MscKeys.AIMIC_ON_PARAM_PREFIX) ){
			boolean ret = false;
			
			AIMic aimic = AIMic.getAIMic();
			if( null != aimic ){
				if( SpeechConstant.IVW_RESET_AIMIC.equals(key) ){
					aimic.reset();
				}else{
					final String realAIMicKey = key.substring( MscKeys.AIMIC_ON_PARAM_PREFIX.length() );
					ret = ErrorCode.SUCCESS==aimic.setParameter( realAIMicKey, value );
				}
				
			}//end of if parameter set right now
			
			return ret;
		}
		
		return super.setParameter(key, value);
	}
	
	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		获取指定的参数的当前值。
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#SAMPLE_RATE}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * 
	 * @param key
	 *            参数名称，所有支持参数，参考{@link #setParameter(String, String)}
	 * @return 参数值，值意义，参考{@link SpeechConstant}对各参数的说明。
	 * 
	 * @see #setParameter(String, String)
	 */
	public String getParameter(String key) {
		// 若是AIMIC即时参数，则通过AIMIC获取
		if( null!=key && key.startsWith(MscKeys.AIMIC_ON_PARAM_PREFIX) ){
			String ret = null;
			
			AIMic aimic = AIMic.getAIMic();
			if( MscKeys.AIMIC_CHANNEL.equals(key) ){
				ret = Integer.toString( AIMic.getChannel() );
			}else if( MscKeys.AIMIC_VERSION.equals(key) ){
				ret = AIMic.getVesion();
			}else if( null != aimic ){
				final String realAIMicKey = key.substring( MscKeys.AIMIC_ON_PARAM_PREFIX.length() );
				ret = aimic.getParameter( realAIMicKey );
			}else{
				DebugLog.LogE( "aimic is null !" );
			}//end of if is aimic parameter
			
			return ret;
		}
		
		return super.getParameter(key);
	}

	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createWakeuper}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isListening}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createWakeuper}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createWakeuper
	 * @see #cancel
	 */
	public boolean destroy() {
		boolean ret = true;
		
		final VoiceWakeuperImpl impl = mWaker;
		if( impl != null ){
			ret = impl.destroy();
		}
		
		final AIMic aimic = AIMic.getAIMic();
		if( null != aimic ){
			aimic.destroy();
		}
		
		if( ret && (ret=super.destroy()) ){
			synchronized( sSync ){
				sInstance = null;
			}//end of sync
			
			//销毁离线引擎和资源
			final SpeechUtility utility = SpeechUtility.getUtility();
			if( null != utility ){
				DebugLog.LogD( "Destory ivw engine." );
				utility.setParameter( ResourceUtil.ENGINE_DESTROY, ResourceUtil.ENGINE_DESTROY+MscKeys.KEY_VAL_SEP+SpeechConstant.ENG_IVW );
			}
		}
		
		return ret;
	}
	
}
