package com.iflytek.cloud.msc.ist;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.Locale;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.record.PcmRecorder;

public class AudioAccessor {
	/**
	 * 类型
	 * @author gxlin
	 *
	 */
	static enum AccesserType{
		WRITE_READ,
		READ_ONLY,
		BUFFER
	}
	
	public static enum AudioKeys{
		CHANNEL
		,FORMAT
		,BIT
		,RATE
	}
	
	public static AudioAccessor createBufferAccessor() throws IOException{
		return new AudioAccessor();
	}
	
	public static AudioAccessor createReadOnlyAccessor( final String filePath )
			throws IOException{
		return new AudioAccessor( filePath );
	}
	
	public static AudioAccessor createWriteReadAccessor( final String filePath, final int sampleRate )
			throws IOException{
		return new AudioAccessor( filePath, sampleRate );
	}
	
	protected AudioAccessor( ) throws IOException{
		mAccesserType = AccesserType.BUFFER;
	}
	
	protected AudioAccessor( final String filePath )
			throws IOException{
		mFilePath = filePath;
		mAccesserType = AccesserType.READ_ONLY;
		
		initFile();
	}
	
	protected AudioAccessor( final String filePath, final int sampleRate ) 
			throws IOException{
		mFilePath = filePath;
		mSampleRate = sampleRate;
		mAccesserType = AccesserType.WRITE_READ;
		
		initFile();
	}
	
	public String getFilePath(){
		String filePath = null;
		
		synchronized ( mFileSyncObj ) {
			if( null != this.mAudioFile ){
				filePath = this.mAudioFile.getAbsolutePath();
			}
		}//end of synchronized
		
		return filePath;
	}
	
	public long getDataLength(){
		long len = 0;
		
		synchronized ( mFileSyncObj ) {
			len = mDataCount;
		}//end of synchronized
		
		return len;
	}
	
	public String getAudioInfo( final AudioKeys key ){
		String val = null;
		switch( key ){
		case CHANNEL:
			val = String.valueOf( this.mNumChannels );
			break;
		case FORMAT:
			val = String.valueOf( this.mFormat );
			break;
		case BIT:
			val = String.valueOf( this.mBitsPerSample );
			break;
		case RATE:
			val = String.valueOf( this.mSampleRate );
			break;
		default:
			break;
		}
		
		return val;
	}
	
	public int getBufferLength(){
		return MAX_BUF_LEN;
	}
	
	public int getCacheLeft(){
		int len = 0;
		
		synchronized(mFileSyncObj){
			//使给外部的缓存是实际缓存的一半，以备事件有冲突时，有足够的缓存
			len = MAX_BUF_LEN/2 - mBufLen;
		}
		
		return len;
	}
	
	public int getAudio( final byte[] buffer ) throws IOException{
		DebugLog.LogS( "getAudioData enter" );
		int bufLen = 0;
		
		if( AccesserType.BUFFER==this.mAccesserType ){
			if( null==buffer || buffer.length!=getBufferLength() ){
				bufLen = -1;
				DebugLog.LogE( "getAudioData buffer is null or length is error !" );
			}else{
				synchronized ( mFileSyncObj ) {
					if( null == mBuffer ){
						throw new IOException( "Data array is null!" );
					}else if( mBufLen > 0 ){
						System.arraycopy( mBuffer, 0, buffer, 0, mBufLen );
						bufLen = mBufLen;
						mBufLen = 0;
						DebugLog.LogS( "getAudioData len:"+bufLen );
					}
				}//end of synchronized
			}
		}else{
			if( null==buffer || buffer.length!=getBufferLength() ){
				bufLen = -1;
				DebugLog.LogE( "getAudioData buffer is null or length is not enough !" );
			}else{
				synchronized( mFileSyncObj ){
					// 从文件中读
					if (null == this.mFileChannel) {
						throw new IOException( "File is null!" );
					}
					
					if( this.getFileLength() > SIZE_OF_WAVE_HEADER ){
						mByteBuffer.clear();
						final int readLen = (int) Math.min( mByteBuffer.capacity(),
								getFileLength() - this.mOffset );
						DebugLog.LogS( "getAudioData buffer len:"+readLen );
						if( readLen > 0 ){
							mFileChannel.position( mOffset );
							if ( readLen != this.readBytes(mOffset, mByteBuffer) ) {
								throw new IOException("Read audio length error:" + readLen);
							}
							
							mByteBuffer.position( 0 );
							mByteBuffer.get( buffer, 0, readLen );
							mOffset += readLen;
							bufLen = readLen;
							DebugLog.LogS( "getAudioData read len:"+readLen );
						}
						
					}//end of if file length
					
				}//end of synchronized
			}
			
		}//end of if-else
		
		DebugLog.LogS( "getAudioData leave" );
		return bufLen;
	}
	
	public synchronized boolean putAudio( byte[] data, final int len ) throws IOException{
		DebugLog.LogS( "putAudio enter" );
		
		if( null == data ){
			DebugLog.LogE( "data is null !" );
			throw new NullPointerException();
		}
		
		if( AccesserType.BUFFER==this.mAccesserType && MAX_BUF_LEN<this.mBufLen+len ){
			DebugLog.LogE( "Buffer is not enough ! "+this.mBufLen );
			throw new IOException( "Buffer is not enough ! "+this.mBufLen );
		}
		
		if( AccesserType.READ_ONLY == this.mAccesserType ){
			DebugLog.LogE( "Current type is "+this.mAccesserType );
			throw new IOException( "Current type is "+this.mAccesserType );
		}
		
		if( null!=data && len>0 ){
			synchronized ( mFileSyncObj ) {
				DebugLog.LogS( "putAudio data len="+len );
				System.arraycopy( data, 0, mBuffer, this.mBufLen, len );
				this.mBufLen += len;
				this.mDataCount += len;
				DebugLog.LogS( "putAudio buf len="+mBufLen );
			}//end of synchronized
		}//end of if data is not null
		
		if( AccesserType.WRITE_READ == mAccesserType ){
			this.saveAudio();
		}
		
		DebugLog.LogS( "putAudio leave" );
		return true;
	}
	
	public synchronized void flush() throws IOException{
		if( AccesserType.WRITE_READ != this.mAccesserType ){
			throw new IOException( "Current type is "+this.mAccesserType );
		}
		
		synchronized ( mFileSyncObj ) {
			mFileChannel.force(true);
			
			this.mLastFlushTime = System.currentTimeMillis();
		}//end of synchronized
	}
	
	public void close() throws IOException{
		DebugLog.LogD( "AudioAccesser close enter" );
		
		synchronized ( mFileSyncObj ) {
			if( AccesserType.WRITE_READ == this.mAccesserType ){
				this.saveAudio();
			}
			
			if( AccesserType.BUFFER != this.mAccesserType ){
				
				if( null != mFileChannel ){
					this.mFileChannel.force( true );
					this.mFileChannel.close();
					this.mFileChannel = null;
				}
				
				if( null != mAccessFile ){
					this.mAccessFile.close();
					this.mAccessFile = null;
				}
			}//end of if accesser type
			
			this.mBuffer = null;
			this.mByteBuffer.clear();
			this.mByteBuffer = null;
		}
		
		DebugLog.LogD( "AudioAccesser close leave" );
	}
	
	private void saveAudio() throws IOException{
		DebugLog.LogS( "saveAudioData enter" );
		
		synchronized (mFileSyncObj) {
			if (null != mFileChannel) {
				DebugLog.LogS("saveAudio write audio len:"
						+ mBufLen+", file length="+getFileLength());
				if( 0 < mBufLen ){
					mByteBuffer.clear();
					final int pos = mByteBuffer.capacity()-mBufLen;
					mByteBuffer.position( pos ); 
					mByteBuffer.put(mBuffer, 0, mBufLen);
					writeBytes((int) getFileLength(), mByteBuffer, pos );
					mBufLen = 0;
					updateAudioFileHeader();
				}

				// 写入磁盘
				if (AccesserType.WRITE_READ == this.mAccesserType
						&& this.isTimeToFlush()) {
					DebugLog.LogS("saveAudio flush to device.");
					flush();
				}//end of if should flush to device
			}//end of if file channel is not null
		}//end of synchronized
		
		DebugLog.LogS( "saveAudioData leave" );
	}
	
	/**
	 * 初始化文件
	 * 因为MscHandler只处理的IOException，所以把FileNotFound转为IOException来处理
	 * @throws IOException
	 */
	private void  initFile() throws IOException{
		if( AccesserType.BUFFER == this.mAccesserType ){
			return;
		}
		
		synchronized( this.mFileSyncObj ){
			if( null == this.mFilePath ){
				throw new IOException( "File path is null" );
			}
			
			if( AccesserType.WRITE_READ == this.mAccesserType ){
				String filePath = mFilePath;
				
				String folderPath = filePath;
				if (filePath.endsWith("/")) {
					folderPath = filePath.substring(0, filePath.lastIndexOf("/") );
				}
				File dir = new File( folderPath );
				if( (!dir.isDirectory()||!dir.exists())
						&& !dir.mkdirs() ){
					IOException exception = new IOException( "create file path failed" );
					throw exception;
				}
				
				//若文件路径不包含文件名，则以当前时间生成文件名
				if( !filePath.endsWith(FILE_FMT) 
						&& !filePath.endsWith(".pcm") ){
					if (!filePath.endsWith("/")) {
						filePath = filePath.concat("/");
					}
					
					Date date = new Date( System.currentTimeMillis() );
					SimpleDateFormat format = new SimpleDateFormat( "yyyyMMdd_HHmmss_SSS", Locale.CHINA );
					final String strTime = format.format(date);
					filePath = filePath.concat(strTime);
					
					mAudioFile = new File(filePath+FILE_FMT);
					int i = 0;
					while(mAudioFile.exists()) {
						++i;
						String newFilePath = filePath + "_" + i + FILE_FMT;
						mAudioFile = new File(newFilePath);
					}

				}else{
					mAudioFile = new File(filePath);
					if( mAudioFile.exists() ){
						throw new IOException( "File is exists:"+filePath );
					}
				}
				
				DebugLog.LogD( "initFile createNewFile:"+filePath );
				if (!mAudioFile.createNewFile()) {
					throw new IOException("create new file \""
							+ mAudioFile.getAbsolutePath() + "\" failed.");
				}
				
				mAccessFile = new RandomAccessFile( mAudioFile, "rw" );
				mFileChannel = mAccessFile.getChannel();
				initAudioFileHeader();
			}else if( AccesserType.READ_ONLY==this.mAccesserType ){
				mAudioFile = new File( this.mFilePath );
				if( !mAudioFile.exists() ){
					throw new IOException( "File is not exist:"+this.mFilePath );
				}
				
				mAccessFile = new RandomAccessFile( mAudioFile, "rw" );
				mFileChannel = mAccessFile.getChannel();
				readAudioInfo();
			}
		}//end of synchronized
		
	}
	
	protected void initAudioFileHeader() throws IOException {

		final String standard = "RIFF";
		final String type = "WAVE";
		final String fmt = "fmt ";
		final String data = "data";
		int offset = 0;

		/* RIFF header */
		writeBytes(offset, standard.getBytes());
		offset += standard.length();

		writeInt(offset, SIZE_OF_WAVE_HEADER);
		offset += 4;

		writeBytes(offset, type.getBytes());
		offset += type.length();

		/** format chunk */
		writeBytes(offset, fmt.getBytes());
		offset += fmt.length();

		writeInt(offset, 16);
		offset += 4;

		writeShort(offset, mFormat);
		offset += 2;

		writeShort(offset, mNumChannels);
		offset += 2;

		writeInt(offset, mSampleRate);
		offset += 4;

		final int transferRate = mNumChannels * mSampleRate * mBitsPerSample
				/ 8;
		final short adjustValue = (short) (mNumChannels * mBitsPerSample / 8);
		DebugLog.LogS("writeAudioFileHeader NumChannels=" + mNumChannels
				+ "SampleRate=" + mSampleRate + ", transferRate="
				+ transferRate + ", adjustValue=" + adjustValue + ", bit="
				+ mBitsPerSample);


		writeInt(offset, transferRate);
		offset += 4;

		writeShort(offset, adjustValue);
		offset += 2;

		writeShort(offset, mBitsPerSample);
		offset += 2;

		/** data chunk */
		writeBytes(offset, data.getBytes());
		offset += data.length();

		writeInt(offset, 0);
	}
	
	protected void readAudioInfo() throws IOException{
		
		final String standard = "RIFF";
		ByteBuffer buffer = ByteBuffer.allocate(4);
		readBytes( 0, buffer );
		String standardVal = new String( buffer.array() );
		if( !standard.equalsIgnoreCase(standardVal) ){
			return;	//若文件头不相同，立即返回
		}
		
		int offset = 20;
		mFormat = readShort( offset );
		
		offset += 2;
		mNumChannels = readShort( offset );
		
		offset += 2;
		mSampleRate = readInt( offset );
		
		offset += 4;
		offset += 4;
		offset += 2;

		mBitsPerSample = readShort( offset );
	}

	protected void writeBytes(int fileOffset,ByteBuffer buffer) throws IOException {
		DebugLog.LogS( "writeBytes buffer len="+buffer.capacity() );
		
		buffer.rewind();
		mFileChannel.position( fileOffset );
		final int len = mFileChannel.write( buffer );
		
		DebugLog.LogS( "writeBytes writen len="+len );
	}
	
	protected void writeBytes( final int fileOffset, final ByteBuffer buffer, final int bufferOffset ) throws IOException{
		DebugLog.LogS( "writeBytes buffer len="+(buffer.capacity()-bufferOffset) );
		
		buffer.position( bufferOffset );
		mFileChannel.position( fileOffset );
		final int len = mFileChannel.write( buffer );
		
		DebugLog.LogS( "writeBytes writen len="+len );
	}
	
	protected void writeBytes( int offset, byte[] buffer ) throws IOException {
		ByteBuffer byteBuffer = ByteBuffer.allocate( buffer.length );
		byteBuffer.put( buffer );
		writeBytes( offset, byteBuffer );
	}

	protected void writeInt(int offset, int val) throws IOException {
		ByteBuffer buffer = ByteBuffer.allocate(4);
		buffer.put( 0, (byte) (val >> 0) );
		buffer.put( 1, (byte) (val >> 8) );
		buffer.put( 2, (byte) (val >> 16) );
		buffer.put( 3, (byte) (val >> 24) );

		writeBytes( offset, buffer );
	}

	protected void writeShort(int offset, short val) throws IOException {
		ByteBuffer buffer = ByteBuffer.allocate(2);
		buffer.put( 0, (byte) (val >> 0) );
		buffer.put( 1, (byte) (val >> 8) );

		writeBytes( offset, buffer );
	}

	protected int readInt( final int offset ) throws IOException{
		ByteBuffer buffer = ByteBuffer.allocate(4);
		mFileChannel.position( offset );
		mFileChannel.read( buffer );
		
		return (int)buffer.getInt(0) << 0
				| (int)buffer.getInt(1) << 8
				| (int)buffer.getInt(2) << 16
				| (int)buffer.getInt(3) << 24;
	}
	
	protected int readBytes(int offset, ByteBuffer buffer) throws IOException {
		mFileChannel.position( offset );
		return mFileChannel.read( buffer );
	}
	
	protected short readShort( final int offset ) throws IOException{
		ByteBuffer buffer = ByteBuffer.allocate(4);
		mFileChannel.position( offset );
		mFileChannel.read(buffer);
		
		return (short)((short)buffer.getShort(0)<<0 | (short)buffer.getShort(1)<<8);
	}
	
	protected void updateAudioFileHeader() throws IOException {
		DebugLog.LogS("updateHeader File length:" + getDataLength()
				+ ", mem file length:" + mFileChannel.size());
		writeInt(FILE_LENGTH_OFFSET, (int) getDataLength());

		DebugLog.LogS("updateHeader data length:"
				+ (getDataLength() - SIZE_OF_WAVE_HEADER));
		writeInt(DATA_LENGTH_OFFSET, (int) getDataLength()
				- SIZE_OF_WAVE_HEADER);
	}

	private boolean isTimeToFlush(){
		return MIN_OUTPUT_TIME<=System.currentTimeMillis()-this.mLastFlushTime;
	}
	
	private long getFileLength() throws IOException{
		int len = 0;
		if( null != this.mFileChannel ){
			len = (int)mFileChannel.size();
		}
		
		DebugLog.LogD( "getFileLength:"+len );
		return len;
	}
	
	/************************************************************/
	private String mFilePath = null;
	private AccesserType mAccesserType = AccesserType.WRITE_READ;
	private int mOffset = 0;
	private static final int MAX_BUF_LEN = 5*1024*1024;	//外部知道的缓存为128K，存是实际缓存的一半，以备事件有冲突时，有足够的缓存
	
	// 音频文件
	private ByteBuffer mByteBuffer = ByteBuffer.allocate( MAX_BUF_LEN );
	private byte[] mBuffer = new byte[MAX_BUF_LEN];
	private int mBufLen = 0; // 需要输出的音频大小
	private int mDataCount = 0; // 累计音频数据大小
	private final static int MIN_OUTPUT_TIME = 3 * 1000; // 最小写入文件间隔
	private long mLastFlushTime = System.currentTimeMillis(); // 上次写入时间
	private File mAudioFile = null; // 音频文件
	private RandomAccessFile mAccessFile = null;
	private FileChannel mFileChannel = null;
	private Object mFileSyncObj = new Object(); // 文件同步对象

	// 音频文件头
	private final String FILE_FMT = ".wav"; // 文件名后缀
	private final int SIZE_OF_WAVE_HEADER = 44; // 文件头大小
	private final int DATA_LENGTH_OFFSET = 40; // 文件头数据长位置
	private final int FILE_LENGTH_OFFSET = 4; // 文件头文件长位置
	private short mNumChannels = 1;
	private short mFormat = 1;
	private short mBitsPerSample = 16;
	private int mSampleRate = PcmRecorder.RATE16K;
	
	/************************************************************/
}
