package com.iflytek.cloud.msc.module;

import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;

import android.content.Context;
import android.media.AudioManager;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import android.os.SystemClock;
import android.util.Log;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.record.PcmRecorder;


public abstract class MscHandler extends Handler {
	protected enum Status {
		init, 		// to start
		start, 		// thread start, to begin session
		recording,  // audio is recording
		waitresult, // waiting for the result,recording is over
		exiting,	// need to exit
		exited,		// session complete
	}
	
	// 消息定义
	// 会话启动消息
	protected final static int MSG_START        = 0;
	// MSC会话启动消息，以便与MSG_START进行分离，做重试操作
	protected final static int MSG_SESSION_BEGIN = 1;
	// 录音数据消息
	protected final static int MSG_RECORD_AUDIO = 2;
	// 录音停止消息，非操作，为通知消息
	protected final static int MSG_RECORD_STOPD = 3;


	// 返回结果消息
	protected final static int MSG_RESULT       = 4;
	// 获取TTS合成音频的消息,需要主动轮询的进行获取
	protected final static int MSG_TTS_AUDIO    = 5;
	// 网络状态值处理
	protected final static int MSG_NET_PERF     = 7;
	// 超时消息
	protected final static int MSG_NET_TIMEOUT  = 8;
	// 录音超时消息，默认60s
	protected final static int MSG_SPEECH_TIMEOUT = 9;
	//上传信息
	protected final static int MSG_UPLOAD       = 10;
	//下载信息
	protected final static int MSG_DOWNLOAD 	= 11;
	//搜索
	protected final static int MSG_SEARCH 	= 12;
	// 对声纹模型的操作请求信息
	protected final static int MSG_ISV_REQUEST 	= 13;
	
	// 清除消息用，清除所有0-CLEAR的消息
	protected final static int MSG_CLEAR        = 20;
	
	// 结束会话消息，需要单独处理，不能被清理掉，防止出现没有sessionEnd
	protected final static int MSG_END          = 21;

	protected final static int MSG_REQUEST_RESULT = 22;
	
	
	// handler任务级别
	protected enum Priority{
		max,
		normal,
//		min
	}
	// 上传流量
	public static final String TAG_UPFLOW = "upflow";
	// 下载流量
	public static final String TAG_DOWNFLOW = "downflow";
	// loginID
	public static final String TAG_LOGIN_ID = "loginid";
	
	// 网络连接状态值，范围[0-100]
	public static final String TAG_NETPERF = "netperf";
	
	// 语音输入超时时间
	protected int          mSpeechTimeOut = 60000;	// default sms 30s,other 7s
	// 录音、播音采样率
	public int          mSampleRate      = PcmRecorder.RATE16K;
	
	
	protected Context 	 mContext = null;
	
	// 会话参数
	private HashParam 	 mParam = new HashParam();
	
	// 用户是否主动退出
	protected volatile boolean  mUserCancel = false;
	
	// 当前会话状态
	private volatile 	 Status mStatus = Status.init;
	
	// 状态开始时间，用以检测是否会超时
	protected long       mStatusBegin  = 0;
	// 网络连接超时时间
	protected int        mNetTimeOut       = 20000;	// default 20s

	private HandlerThread mThread;

	
	protected static final HashSet<MscHandler> sInstances = new HashSet<MscHandler>();
	
	public MscHandler(Context context,HandlerThread handlethread){
		super( handlethread.getLooper() );
		mThread = handlethread;
		mContext = context;
		mUserCancel = false;
		
		sInstances.add( this );
	}
	public MscHandler(Context context){
		super(context.getMainLooper());
		mContext = context;
		mUserCancel = false;
	}
	
	/**
	 * 消息处理函数
	 * @param msg
	 * @throws Throwable TODO
	 * @throws SpeechError TODO
	 */
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError{
	}
	
	/**
	 * 更新超时处理的消息，在进入某一状态时进行更新
	 * 识别：进入等待结果状态，或者获取到音频
	 * 合成：sessionBegin成功后，或者获取到音频
	 */
	protected void updateTimeoutMsg()
	{
		removeMessages(MSG_NET_TIMEOUT);
		sendMsg(MSG_NET_TIMEOUT, Priority.normal, false, mNetTimeOut);
	}
	private void destroyThread() {
		if( null!=mThread && mThread.isAlive() ){
			clearAllMsg();
			
			Thread mainThread = null;
			Looper mainLooper = null;
			if( null != mContext && null!=(mainLooper=mContext.getMainLooper()) ){
				mainThread = mainLooper.getThread();
			} 
			
			if( null==mContext || !mThread.equals(mainThread) ){
				mThread.quit();
				DebugLog.LogD( "quit current Msc Handler thread" );
			}
			
			mThread = null;
		}
		
		sInstances.remove( this );
	}
	
	/**
	 * 消息处理模型
	 * 如果是完成结束消息，在onEnd接口中进行处理，调用EnssionEnd；
	 * 如果是其他消息类型，在业务类中进行处理，Handler本身负责捕获异常，进入exit消息
	 */
	public void handleMessage( Message msg )
	{
		switch (msg.what) {
		case MSG_END:
			onEnd((SpeechError)msg.obj);
			
			//关闭线程
			destroyThread();
			break;
		default:
			SpeechError error = null;
			try {
				// 会话消息处理,在此处理通用消息部分
				switch (msg.what) {
				case MSG_NET_TIMEOUT:
					throw new SpeechError(ErrorCode.ERROR_NETWORK_TIMEOUT);
				default:
					if( null==SpeechUtility.getUtility() && MSG_SESSION_BEGIN==msg.what ){
						DebugLog.LogE( "SDK is not init while session begin" );
						throw new SpeechError(ErrorCode.ERROR_LOGIN);
					}
					onMsgProcess(msg);
					break;
				}
			} catch (IOException e) {
				DebugLog.LogE(e);
				error = new SpeechError(ErrorCode.ERROR_FILE_ACCESS);
			} catch (SpeechError e) {
				DebugLog.LogE(e);
				error = e;
			} catch (Exception e) {
				DebugLog.LogE(e);
				error = new SpeechError(e);
			}catch( UnsatisfiedLinkError e ){
				DebugLog.LogE( e );
				error = new SpeechError( ErrorCode.ERROR_UNSATISFIED_LINK );
			} catch( Throwable e ){
				DebugLog.LogE( e );
				error = new SpeechError( ErrorCode.ERROR_UNKNOWN );
			}finally {
				// 进入错误处理流程
				if(error != null)
				{
					DebugLog.LogD(getTag() + " occur Error = " + error.toString());
					exit(error);
				}
			}
			break;
		}
	}
	
	/**
	 * 是否长时语音输入，sms、cantonese为30s,poi、video为短时7s
	 * @return
	 */
	public boolean isLongInput()
	{
		return false;
	}
	
	/**
	 * 请求参数的编码类型
	 * 会话参数编码类型
	 * @return
	 */
	public String getParamEncoding()
	{
		return mParam.getString(MscKeys.KEY_PTE, DataUtil.UTF8);
	}
	/**
	 * 会话上传文本的编码
	 * 文字编码类型，表示上传文字编码，不是参数编码
	 * @return
	 */
	public String getTextEncoding()
	{
		return mParam.getString(SpeechConstant.TEXT_ENCODING, DataUtil.UTF8);
	}
	
	/**
	 * 请求文本的编码类型
	 * 文字编码类型，表示上传文字编码，不是参数编码
	 * @return
	 */
	public String getResultEncoding()
	{
		return mParam.getString(MscKeys.KEY_RSE, DataUtil.UTF8);
	}
	
	/**
	 * 获取录音、播音的采样率
	 * @return 示例：16000、8000
	 */
	public int getSampleRate()
	{
		return mSampleRate;
	}
	
	/**
	 * 取消当前会话,清除队列消息，发送结束通知
	 */
	public void cancel(boolean notifyError) {
		mUserCancel = true;
		// 清除队列消息
		clearAllMsg();
		exit(null);
	}
	/**
	 * 发送结束消息，停止会话
	 * @param error
	 */
	protected synchronized void exit(SpeechError error) {
		// 如果发生错误，发送错误消息，清除队列
		if(error != null)
			clearAllMsg();

		// 发送完成消息
		sendMsg( obtainMessage(MSG_END, error) );
	}
	
	public static boolean isEmpty(){
		return sInstances.isEmpty();
	}
	
	/**
	 * 是否处于运行状态
	 * @return
	 */
	public boolean isRunning() {
		if(mStatus == Status.exited || mStatus == Status.exiting
				|| mStatus == Status.init)
			return false;
		return true;
	}
	/**
	 * 改变识别状态，需要防止mStatus=exiting状态被误改变
	 * @param _status
	 */
	protected synchronized void setStatus(Status _status) {
		DebugLog.LogD("curStatus=" + mStatus + ",setStatus=" + _status);
		if(mStatus == Status.exited)
			return;
		if(mStatus == Status.exiting && _status != Status.exited)
			return;
		DebugLog.LogD("setStatus success=" + _status);
		mStatus = _status;
		mStatusBegin = SystemClock.elapsedRealtime();
	}
	
	/**
	 * 返回当前会话状态
	 * @return
	 */
	protected synchronized Status getStatus()
	{
		return mStatus;
	}
	/**
	 * 返回服务端SessionID
	 * 服务端交互的sessionID，区别于IsrSession中使用的本地sessionID
	 */
	public abstract String getSessionID();
	/**
	 * 获取客户端ID
	 * 客户端本地生成的会话ID
	 */
	public abstract String getClientID();
	
	/**
	 * 保存会话参数
	 * @param params
	 */
	protected void setParams(HashParam params) {
		mParam = params.clone();
		onParseParam();
	}
	
	/**
	 * 解析会话参数,调用setParams接口时进行调用
	 */
	protected void onParseParam()
	{
		// 网络超时时间
		mNetTimeOut = mParam.getInt(SpeechConstant.NET_TIMEOUT, mNetTimeOut);
		// 采样率
		mSampleRate = mParam.getInt(SpeechConstant.SAMPLE_RATE, mSampleRate);
	}
	
	/**
	 * 获取会话参数
	 * @return
	 */
	public HashParam getParam() {
		return mParam;
	}
	/**
	 * 开始会话,发送start消息
	 */
	protected void start() {
		/*try{
			mCollector.onSessionBegin();
		}catch( Throwable e ){
			DebugLog.LogE( "DC exception:" );
			DebugLog.LogE( e );
		}*/
		
		sendMsg(MSG_START,Priority.max,false,0);
	}
	
	/**
	 * 发送消息处理
	 * @param what 消息索引
	 */
	protected void sendMsg(int what)
	{
		// 使用下方接口映射
		sendMsg( obtainMessage(what),Priority.normal,false ,0);
	}
	/**
	 * 发送消息处理
	 * @param msg
	 */
	protected void sendMsg(Message msg)
	{
		// 使用下方接口映射
		sendMsg( msg,Priority.normal,false,0);
	}
	/**
	 * 发送消息处理，增加优先级、是否清除参数
	 * @param what 消息索引
	 * @param priority 级别
	 * @param clean 是否clean
	 * @param delay 延迟时间
	 */
	protected void sendMsg(int what,Priority priority,boolean clean,int delay)
	{
		// 使用下方接口映射
		sendMsg(obtainMessage(what),priority,clean,delay);
	}
	
	/**
	 * 消息处理方法，修改状态值
	 * 如果是录音消息，在SessionBegin未处理情况下，需要先缓存起来
	 * @param msg
	 * @param priority
	 * @param clean
	 */
	protected void sendMsg(Message msg,Priority priority,boolean clean,int delay){
		
		// 如果会话已经结束，不接收消息
		if(getStatus() == Status.exited
		  || getStatus() == Status.exiting){
			DebugLog.LogD( "send msg failed while status is "+getStatus() );
			return;
		}
		
		// 修改状态值
		switch (msg.what) {
		case MSG_START:
			setStatus(Status.start);
			break;
		case MSG_RECORD_STOPD:
			setStatus(Status.waitresult);
			break;
		case MSG_END:
			setStatus(Status.exiting);
			break;
		default:
			break;
		}
		
		if( clean ){
			this.removeMessages( msg.what );
		}
		
		// 插入消息
		if(priority == Priority.max && delay <= 0)  // 不需要处理延迟时间
			sendMessageAtFrontOfQueue(msg);
		else
			sendMessageDelayed(msg, delay);
	}
	/**
	 * 清除所有队列消息
	 */
	protected void clearAllMsg()
	{
		DebugLog.LogD( "clear all message" );
		for(int i= 0; i < MSG_CLEAR; i++)
			removeMessages(i);
	}
	/**
	 * 会话结束通知外部消息
	 * 如果修改方法内容，请考虑MixMscRecognizer
	 */
	protected void onEnd(SpeechError error) {
		setStatus(Status.exited);
		clearAllMsg();
		// 统计分析，收集上传用户数据
		//UserLogger userlog = UserLogger.getUserLogger(mContext);
		//userlog.collectAndUploadProcess();

	}

	protected String getTag()
	{
		return this.getClass().toString();
	}
	
	public static void timeOutCheck(long startTime,int outTime) throws SpeechError
	{
		if(SystemClock.elapsedRealtime()-startTime > outTime)
			throw new SpeechError(ErrorCode.ERROR_NETWORK_TIMEOUT);
	}

	/**
	 * Requires Permission:
	 * 		android.permission.MODIFY_AUDIO_SETTINGS
	 * 		android.permission.BLUETOOTH 
	 * @return
	 */
	public void startBluetooth() throws SecurityException{
		DebugLog.LogD( "startBluetooth enter" );

		AudioManager mAudioManager = (AudioManager)mContext.getSystemService(Context.AUDIO_SERVICE);
		mAudioManager.setBluetoothScoOn(true);
		mAudioManager.startBluetoothSco();
	}
	
	public void stopBluetooth(){
		DebugLog.LogD( "stopBluetooth enter" );
		try{
			AudioManager mAudioManager = (AudioManager)mContext.getSystemService(Context.AUDIO_SERVICE);
			mAudioManager.setBluetoothScoOn(false);
			mAudioManager.stopBluetoothSco();
		}catch( SecurityException e ){
			DebugLog.LogE( e );
		}catch( Throwable e ){
			DebugLog.LogE( e );
		}
		
	}
}
