package com.iflytek.cloud.msc.module;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;

import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechListener;
import com.iflytek.cloud.msc.util.Config;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.record.PcmRecorder;

/**
 * 所有Speech对象均按照单体模式进行设计，不支持多路并发
 * @author yjzhao
 *
 */
public abstract class SpeechModule extends SpeechInterface{
	//Application context
	protected Context mContext = null;
	//Object Lock
	protected Object mSynObj = new Object();
	
	/*
	 * Current active session object, used to determine whether or not
	 * can make a new session.
	 */
	protected volatile MscHandler mscer = null;
	
	//会话执行的线程HandlerThread
	protected volatile HandlerThread mscht = null;
	
	/**
	 * 初始化会话对象.
	 * @param context
	 */
	protected SpeechModule(Context context) {
		
		if(context != null)
		{
			Config.getConfig(context.getApplicationContext());
			mContext = context.getApplicationContext();
			try {
				onInit();
			} catch (Exception e) {
				DebugLog.LogE(e);
			}
		}
		else
			mContext = null;
	}
	/**
	 * 实例化相应的handlerthread
	 * @param name handlerthread名字
	 * @return
	 */
	protected HandlerThread getHandlerThread(String name) throws Throwable{
		mscht = new HandlerThread( name );
		mscht.start();

		return mscht;
	}
	/**
	 * virtual function,to init msc 
	 */
	protected void onInit() throws Exception{}
	
	/**
	 * get class name for logcat
	 * @return
	 */
	protected String getTag()
	{
		return this.getClass().toString();
	}	
	
	/**
	 * According to current session state to determine whether or not can 
	 * open a new session.
	 * 
	 * @return weather running state
	 */
	protected boolean isRunning()
	{
		if(mscer != null && mscer.isRunning())
			return true;
		return false;
	}
	
	/**
	 * 取消当前会话
	 */
	public void cancel(boolean notifyError) {
		if(mscer != null)
		{
			mscer.cancel(notifyError);
		}
	}

	/**
	 * 获取录音、播音采样率
	 * @return 录音、播音采样率
	 */
	public int getSampleRate()
	{
		return  mSessionParams.getInt(SpeechConstant.SAMPLE_RATE, PcmRecorder.RATE16K);
	}
	
	/**
	 * 销毁语音对象
	 * @return true表示销毁成功，false表示销毁失败（会话正在进行中）
	 */
	public boolean destroy() {
		boolean ret = true;
		
		synchronized (mSynObj) {
			if(isRunning()){
				mscer.cancel(false);
				ret = false;
			}else{
				ret = onFini();
				DebugLog.LogS(getTag() + "destory =" + ret);
			}	
		}//end of sync
		
		if( ret ){
			ret = super.destroy();
		}
		
		return ret;
	}
	
	/**
	 * 销毁，逆初始化
	 */
	protected boolean onFini() {
		//destory方法不可以，单纯的stop可能会存在隐患
		if(null != mscht && mscht.isAlive()){
			HandlerThread tempthread = mscht;
			mscht = null;
		    tempthread.interrupt();
		}
		return true;
	}
	
	@Override
	protected void finalize() throws Throwable {
		DebugLog.LogD(getTag() +  " finalize called");
		super.finalize();
	}
	
	
	private static final int BASE_MSG_EVENT = 0;
	private static final int BASE_MSG_DATA = 1;
	private static final int BASE_MSG_END = 2;
	/**
	 * 通用消息回调接口，通过MainLooper转发到主线程
	 * @author yjzhao
	 *
	 */
	protected class InnerSpeechListener implements SpeechListener {
		private SpeechListener mOutListener = null;
		public InnerSpeechListener(SpeechListener outlistener) {
			mOutListener = outlistener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				DebugLog.LogD("SpeechListener onMsg = " + msg.what);
				switch (msg.what) {
				case BASE_MSG_EVENT:
					mOutListener.onEvent(msg.arg1,(Bundle)msg.obj);
					break;
				case BASE_MSG_DATA:
					mOutListener.onBufferReceived((byte[])msg.obj);
					break;
				case BASE_MSG_END:
					mOutListener.onCompleted((SpeechError) msg.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};

		@Override
		public void onBufferReceived(byte[] buffer) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_DATA,buffer);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEvent(int eventType, Bundle params) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_EVENT,eventType,0,params);
			mUiHandler.sendMessage(msg);
		}
		
		@Override
		public void onCompleted(SpeechError error) {
			Message msg = mUiHandler.obtainMessage(BASE_MSG_END, error);
			mUiHandler.sendMessage(msg);
			
		}
	}
}
