package com.iflytek.cloud.speech.impl;

import java.io.UnsupportedEncodingException;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechListener;
import com.iflytek.cloud.TextUnderstanderListener;
import com.iflytek.cloud.UnderstanderResult;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.msp.MscCommon;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.msc.MSC;

/**
 * 文本转语义类。实例化该对象之前需要先调用{@link 
 * com.iflytek.cloud.SpeechUtility#createUtility(Context, String)}
 * 进行初始化。<br>
 * @author <a href="http://www.xfyun.cn/">讯飞语音云</a>
 *
 */
public class TextUnderstanderImpl extends SpeechModule{
	
	/**
	 * 构造函数，不进行任何操作。<br>
	 */
	public TextUnderstanderImpl(Context context) {
		super(context);
	}
	
	/**
	 * 文本转语义接口，输入文本内容，获取语义理解结果。<br>
	 * @param text 需要理解的文本
	 * @param listener 监听器
	 * @return 
	 */
	public int understandText(String text, TextUnderstanderListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		try{
			if(TextUtils.isEmpty(getParameter(SpeechConstant.ASR_SCH))) 
				setParameter(SpeechConstant.ASR_SCH, "1");
			if(TextUtils.isEmpty(getParameter(SpeechConstant.NLP_VERSION))){
				final String defNlpVersion = MSC.isIflyVersion() ? "3.0" : "2.0";
				setParameter(SpeechConstant.NLP_VERSION, defNlpVersion);
			}
			if(TextUtils.isEmpty(getParameter(SpeechConstant.RESULT_TYPE)))
				setParameter(SpeechConstant.RESULT_TYPE, "json");
			
			//MSC为同步接口，屏蔽外部重复调用导致的多线程卡死
			if(isRunning()){
				ret = ErrorCode.ERROR_ENGINE_BUSY;
			}else{
				mscer = new MscCommon(mContext,mSessionParams,getHandlerThread( "textunderstand" ));
				SelfUnderstanderListener searchListener = new SelfUnderstanderListener(listener);
				((MscCommon)mscer).startUnderstandText(new InnerSpeechListener(searchListener),text);
			}
		}catch(SpeechError e){
			ret = e.getErrorCode();
			DebugLog.LogE( e );
		}catch( Throwable e ){
			ret = ErrorCode.ERROR_UNKNOWN;
			DebugLog.LogE( e );
		}//end of try-catch
		
		return ret;
	}

	/**
	 * 是否正在语义处理。<br>
	 * @return true表示正在进行语义识别，false表示空闲
	 */
	public boolean isUnderstanding() {
		return isRunning();
	}
	
	/**
	 * 取消语义处理。
	 */
	@Override
	public void cancel(boolean notifyError) {
		super.cancel(notifyError);
	}

	/**
	 * 语义识别参数设置，key可设置的参数有：<br>
	 * SpeechConstant.NET_TYPE:网络类型 <br>
	 * SpeechConstant.NET_TIMEOUT:网络连接超时时间 <br>
	 * SpeechConstant.LANGUAGE:语言<br>
	 * SpeechConstant.ACCENT:语言区域<br>
	 * SpeechConstant.DOMAIN:应用领域<br>
	 * SpeechConstant.RESULT_TYPE:识别结果类型<br>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return 错误码，0表示成功
	 */
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}

	/**
	 * 获取语义参数。
	 * <p>包括引擎类型、语言、语言区域、领域、前后端点超时等，另外包括支持的语言列表等。
	 * @param key 参数名称
	 * @return 参数值
	 */
	public String getParameter(String key) {
		return super.getParameter(key);
	}

	/**
	 * 销毁识别对象
	 */
	@Override
	public boolean destroy() {
		return super.destroy();
	}
	
	/**
	 * 语义理解回调监听
	 *
	 */
	 private class SelfUnderstanderListener implements SpeechListener {
		 
    	// 外部监听
        private TextUnderstanderListener mListener;

        public SelfUnderstanderListener(TextUnderstanderListener listener) {
			mListener = listener;
		}
        
		public void onBufferReceived(byte[] data) {
			try {
				if(data != null)
				{
					UnderstanderResult result = new UnderstanderResult(
							new String(data,DataUtil.UTF8));
					mListener.onResult(result);
				}
			} catch (NullPointerException e) {
                DebugLog.LogE(e);
            } catch (UnsupportedEncodingException e) {
				DebugLog.LogE(e);
			}
			
		}

		@Override
		public void onCompleted(SpeechError error) {
        	if(mListener != null && error != null)
        		mListener.onError(error);       			
		}

		@Override
		public void onEvent(int arg0, Bundle arg1) {
			
		}
		 
	 }
}
