package com.iflytek.cloud.util;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

/**
 * <h3>重力传感器</h3>
 *
 * 	用于开启重力传感器，以获得当前手机朝向，用于人脸检测时，获取照片的正确方向。
 *
 * 
 * @author
 * @since version:1073
 * @see com.iflytek.cloud.FaceDetector
 * @see com.iflytek.cloud.FaceRequest
 * @see com.iflytek.cloud.IdentityVerifier
 */
public class Accelerometer {
	/**
	 */
	public enum CLOCKWISE_ANGLE {
		Deg0(0), Deg90(1), Deg180(2), Deg270(3); 
		private int value;
		private CLOCKWISE_ANGLE(int value){
			this.value = value;
		}
		
		/**
		 *
		 *
		 * 获取枚举成员对应的方向int型值，如Deg90，对应值为1。
		 *
		 * @return 枚举成员对应的方向int型值
		 */
		public int getValue() {
			return value;
		}
	}

	private SensorManager sensorManager = null;

	private boolean hasStarted = false;
	
	private static CLOCKWISE_ANGLE rotation;

	/**
	 * <h4>构造函数</h4>
	 *
	 * 		构造一个实例。
	 *
	 * @param ctx 应用上下文
	 */
	public Accelerometer(Context ctx) {
		sensorManager = (SensorManager) ctx
				.getSystemService(Context.SENSOR_SERVICE);
		rotation = CLOCKWISE_ANGLE.Deg0;
	}
	
	/**
	 * <h4>开始监听传感器</h4>
	 * <p>
	 * 		调用此函数，开始监听重力传感器。监听开始后，可通过{@link #getDirection}获取
	 * 当前手机的转向。通过{@link #stop}停止监听。
	 * </p>
	 * 
	 * @see #stop
	 * @see #getDirection
	 */
	public void start() {
		if (hasStarted) return;
		hasStarted = true;
		rotation = CLOCKWISE_ANGLE.Deg0;
		sensorManager.registerListener(accListener,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_NORMAL);
	}

	/**
	 * <h4>结束传感器监听</h4>
	 * <p>
	 * 		结束监听重力传感器后，无法再获取当前手机的转向。可通过{@link #start}再次
	 * 开启重力传感器监听。
	 * </p>
	 * 
	 * @see #start
	 */
	public void stop() {
		if (!hasStarted) return;
		hasStarted = false;
		sensorManager.unregisterListener(accListener);
	}

	/**
	 * <h4> 返回当前手机转向</h4>
	 * <p>
	 * 		调用此函数，获取当前手机的转向，返回的值为{@link CLOCKWISE_ANGLE#getValue()}
	 * 的值。通过此函数获取转向前，应该先通过{@link #start}开始监听。
	 * </p>
	 *@return 转向值，参考{@link CLOCKWISE_ANGLE#getValue()}的值
	 *
	 *@see #start
	 *@see CLOCKWISE_ANGLE#getValue
	 */
	static public int getDirection() {
		return rotation.getValue();
	}

	/**
	 * 传感器与手机转向之间的逻辑
	 */
	private SensorEventListener accListener = new SensorEventListener() {

		@Override
		public void onAccuracyChanged(Sensor arg0, int arg1) {
		}

		@Override
		public void onSensorChanged(SensorEvent arg0) {
			if (arg0.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				float x = arg0.values[0];
				float y = arg0.values[1];
				float z = arg0.values[2];
				if (Math.abs(x)>3 || Math.abs(y)>3) {
					if (Math.abs(x)>Math.abs(y)) {
						if (x > 0) {
							rotation = CLOCKWISE_ANGLE.Deg0;
							//Log.d("ROTATION","CLOCKWISE_ANGLE: Deg0");
						} else {
							rotation = CLOCKWISE_ANGLE.Deg180;
							//Log.d("ROTATION","CLOCKWISE_ANGLE: Deg180");
						}
					} else {
						if (y > 0) {
							rotation = CLOCKWISE_ANGLE.Deg90;
							//Log.d("ROTATION","CLOCKWISE_ANGLE: Deg90");
						} else {
							rotation = CLOCKWISE_ANGLE.Deg270;
							//Log.d("ROTATION","CLOCKWISE_ANGLE: Deg270");
						}
					}
				}
			}
		}
	};
}
