package com.iflytek.cloud.util;

import java.util.LinkedHashMap;

import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.util.code.impl.AudioDetectorImplFixfrontVad;
import com.iflytek.cloud.util.code.impl.AudioDetectorImplMetaVad;

import android.content.Context;
import android.text.TextUtils;

/**
 * <h3>音频检测类</h3>
 * <p>	此功能暂未支持。</p>
 * <p>
 * 		通过音频检测类，检测音频中，指定静音长度出现的位置，音频音量、质量。
 * 通过此类，应用层可以通过指定端点长度，把音频切分（在返回时结果出现端
 * 点时就认为是一小段）。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createDetector}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getDetector}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createDetector}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link com.iflytek.cloud.SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1083
 * @author iFlytek &nbsp;&nbsp;&nbsp;<a href="http://www.xfyun.cn/">
 * 				讯飞语音云</a>
 * 
 */
public abstract class AudioDetector {
	/**
	 * <h3>检测结果类</h3>
	 * <p>
	 * 		检测音频时，检测结果。
	 * </p>
	 */
	public static class DetectorResult{
		/**
		 * <h4>音频数据缓存</h4>
		 * <p>
		 * 		存放音频数据的内存空间。
		 * </p>
		 * <p>
		 * 		此结果值，仅在{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为 {@link AudioDetector#TYPE_FIXFRONT}时生效。
		 * </p>
		 * <p>值类型：byte[]</p>
		 * <p>值域：{null, 对应的数据数组}</p>
		 */
		public  byte[] buffer = null;
		
		/**
		 * <h4>数据偏移</h4>
		 * <p>
		 * 		指定{@link #buffer}有效数据开始位置，相对当前数组开始位置的偏移大小。
		 * </p>
		 * <p>
		 * 		此结果值，仅在{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为 {@link AudioDetector#TYPE_FIXFRONT}时生效。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：[0, +∞)</p>
		 */
		public int offset = 0;
		
		/**
		 * <h4>数据长度</h4>
		 * <p>
		 * 		数据长度，指定{@link #buffer}中有效的数据的长度。
		 * length <= {@link #buffer}.length - offset。
		 * </p>
		 * <p>
		 * 		此结果值，仅在{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为 {@link AudioDetector#TYPE_FIXFRONT}时生效。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：[0, +∞)</p>
		 */
		public int length = 0;
		
		/**
		 * <h4>状态</h4>
		 * <p>
		 * 		标记是否检测到前端点或后端点。
		 * <br>{@link #STATUS_OK}：没有检测到端点（正在检测）;
		 * <br>{@link #STATUS_START}：检测到前端点；
		 * <br>{@link #STATUS_EOS}：检测到后端点，或{@link AudioDetector#SUB_TIMEOUT 分句超时}（分句超时
		 * 情况仅在{@link AudioDetector#VAD_ENGINE 引擎类型}为
		 * {@link AudioDetector#TYPE_FIXFRONT}时出现）；
		 * <br>{@link #STATUS_BOS}：检测到前端点超时；
		 * <br>{@link #STATUS_TIMEOUT}：检测到音频超时；（从检测到前端点开始算起，写入的音频时间长度超过
		 * {@link com.iflytek.cloud.SpeechConstant#KEY_SPEECH_TIMEOUT}指定的值时，还没
		 * 检测到后端点，视为音频超时。与{@link AudioDetector#SUB_TIMEOUT 分句超时}的区别在于，
		 * 分句超时以{@link #sub 分句}长度为判断点，而音频超时，则以前端点后的所有音频长度为准。）
		 * </p>
		 * <p>
		 * 		状态值的返回，约有250ms左右音频长度的延时——即若在当前调用{@link AudioDetector#detect}
		 * 的返回结果中，获取到状态值，则实际音频中该状态值位置可能在当前位置前的[0, 250]ms长度中。
		 * 当连续的检测一段音频，在找到一个前端点(start_1)和后端点(end_1)后，从在第二次前、后端点(start_2)状态返回时，
		 * 很可能有 250ms*2 = 500ms 的位置延迟——在end_1时已经延迟了250ms，而在 start_2 时，又再延迟了 250ms。
		 * 故若需要实时知道端点位置，则实际前、后端点应该为<br>
		 *  startOffset（或 endOffset） = max( lastEndOffset, curOffset-maxDelayTime*bytePerMs )，<br>
		 * 其中，lastEndOffset 为 前一次找到的后端点位置的偏移字节数；curOffset 指返回端点时，当前写入音频的偏移位置；bytePerMs 指每毫秒
		 * 的字节数，maxDelayTime = min( {@link com.iflytek.cloud.SpeechConstant#VAD_EOS}, 500 ) 。取 vad_eos 与
		 * 500 中的小值，是为了避免向前偏移状态返回的延迟时，把上一次的后端点非静音的音频也划分到新的切分段中——因此，建议
		 * 在检测时，{@link com.iflytek.cloud.SpeechConstant#VAD_EOS} 的值设置应该大于500。
		 * 若不需要实时知道前端点，则可在返回后端点时，直接通过{@link #start}获取前端点位置。
		 * </p>
		 * <p>
		 * 		在检测到后端点，或前端点超时时，可以考虑通过判断音频文件在后端点后面的长度小于正常
		 * 说话的有效音频长度时，如1秒，直接把端点后面的音频忽略，不必再检测。
		 * </p>
		 * <p>
		 * 		在找后端点，并切分音频为长度上限为某个值时（如一次听写支持的最大长度），
		 * 可以通过计算当前已经写入SDK的音频长度，是否已达到上限值，当达到上限，还
		 * 未找到后端点时，可以考虑优先使用前一个{@link #sub 分句}结束
		 * 位置（假设存在时）作为切分点，然后再从该位置，开始一次新的检测，找下一个
		 * 切分点。
		 * </p>
		 * <p>
		 * 		当检测到前端点超时，或后端点时，说明截止当前返回的音频为止，已检测到一段满足
		 * 前后端点静音超时长度，或分句超时（仅在引擎为
		 * {@link AudioDetector#TYPE_FIXFRONT}时）的音频，一次检测结束。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：{0 , 1, 2, 3, 4}</p>
		 */
		public int status = 0;
		
		/**
		 * <h4>分句状态信息</h4>
		 * <p>
		 * 		分句信息标记音频是否检测到一段话里的分句信息：
		 * <br>{@link #SUB_OK}：没有检测到分句信息；
		 * <br>{@link #SUB_START}：检测到分句开始；
		 * <br>{@link #SUB_END}：检测到分句结束；
		 * <br>{@link #SUB_STARTEND}：检则到分句开始和结束；
		 * </p>
		 * <p>
		 * 		分句，指界于前后端点间，音频句子停顿的地方。可以把前
		 * 后端点视作文章段落位置，而分句开始和结束，则是段落中的
		 * 逗号和句号的位置。
		 * </p>
		 * <p>
		 * 		当{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为{@link AudioDetector#TYPE_FIXFRONT}时，返回
		 * 实时的分句信息，应用层通过当前写入SDK的音频位置信
		 * 息来划分分句。
		 * </p>
		 * <p>
		 * 		当{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为{@link AudioDetector#TYPE_META}时，除了首个分
		 * 句开始会实时返回外，其他分句将返回非实时的
		 * 分句信息，应用层通过 {@link #subs}
		 * 来获取分句的开始和结束位置信息，以划分分句。且，此时只会出
		 * 现0或3的分句信息——即，分句开始和结束总是同时返回的。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：{0 , 1, 2, 3}</p>
		 */
		public int sub = 0;
		
		/**
		 * <h4>分句位置信息</h4>
		 * <p>
		 * 		仅在{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为{@link AudioDetector#TYPE_META}时生效。
		 * </p>
		 * <p>
		 * 		{@link AudioDetector#TYPE_META}模式下，当获取到分句
		 * 信息时，有可能有多个分句开始和结束信息，以 key、value
		 * 的形式，顺序的存放在 LinkedHashMap 的元素中。
		 * </p>
		 * <p>值类型：Map<Integer, Integer></p>
		 * <p>值域：元素个数为[0, +∞)个</p>
		 */
		final public LinkedHashMap<Integer, Integer> subs = new LinkedHashMap<Integer, Integer>();
		
		/**
		 * <h4>音量值</h4>
		 * <p>
		 * 		实时返回当前音频数据的音量值。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：[0, 30]</p>
		 */
		public int volume = 0;
		
		/**
		 * <h4>是否有语音</h4>
		 * <p>
		 * 		当前返回的音频数据中，是否含语音。
		 * </p>
		 * <p>
		 * 		此结果值，仅在{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为 {@link AudioDetector#TYPE_FIXFRONT}时生效。
		 * </p>
		 * <p>值类型：boolean</p>
		 * <p>值域：{true , false}</p>
		 */
		public boolean voice = false;
		
		/**
		 * <h4>质量值</h4>
		 * <p>
		 * 		当前返回音频数据的质量值。
		 * <br>正常：0；
		 * <br>前截断：1；
		 * <br>后截断：2；
		 * <br>前后均截断：3；
		 * <br>背景噪音大：4；
		 * <br>背景有音乐等：5；（暂不支持）
		 * <br>声音太大而截幅：6；
		 * <br>声音过高：7；
		 * <br>声音过低：8；
		 * </p>
		 * <p>
		 * 		仅在{@link #status}非0时出现。
		 * </p>
		 * <p>
		 * 		此结果值，仅在{@link AudioDetector#VAD_ENGINE 引擎类型}
		 * 为 {@link AudioDetector#TYPE_FIXFRONT}时生效。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：[0, 8]</p>
		 */
		public int quality = 0;
		
		/**
		 * <h4>语音开始位置</h4>
		 * <p>
		 * 		此结果值正在优化，建议获取有效音频的起始、结束点 通过端点状态
		 * 返回时来计算，参考 {@link #status}}。
		 * </p>
		 * <p>
		 * 		语音开始位置，指在检测到后端点时，有语音的音频数据开始位置，与在
		 * {@link AudioDetector#reset()}调用后（首次检测时可以不调用）后，写入的
		 * 音频字节数开始算起的偏移字节。
		 * </p>
		 * <p>
		 * 		需要注意的是，如果是连续的检测一段音频，在第二次的后端点结果返回时，
		 * 因为第一次返回后端点结果的时，有延迟（参考{@link #status}的说明），当
		 * 计算在实际音频中的偏移时，应该减去这个延迟的偏移，如：<br>
		 *  startOffset = start + max( lastEndOffset, bufStart-maxDelayTime*bytePerMs ); <br>
		 *  其中，maxDelayTime = min( {@link com.iflytek.cloud.SpeechConstant#VAD_EOS}, 500 )，
		 *  bytePerMs指每毫秒的字节数，bufStart 为最近一个{@link AudioDetector#reset()}调用后的字
		 *  节偏移。lastEndOffset 为 前一次找到的后端点位置的偏移字节数；
		 * </p>
		 * <p>另注意{@link AudioDetector#RESET_SENTENCE} 的说明。</p>
		 * <p>
		 * 		仅在{@link #status}为1或2时出现。
		 * </p>
		 * <p>值类型：int</p>
		 * <p>值域：[0, +∞)</p>
		 * 
		 * @see AudioDetector#RESET_SENTENCE
		 */
		public int start = 0;
		
		/**
		 * <h4>语音结束位置</h4>
		 * <p>
		 * 		此结果值正在优化，建议获取有效音频的起始、结束点 通过端点状态
		 * 返回时来计算，参考 {@link #status}}。
		 * </p>
		 * <p>
		 * 		语音结束位置，指在检测到后端点时，有语音的音频数据结束位置，相对
		 * {@link AudioDetector#reset()} 调用后（首次检测时可以不调用）后，写入的
		 * 音频字节数开始算起的偏移字节。 即有{@link #start} <= {@link #end} 。
		 * </p>
		 * <p>
		 * 		需要注意的是，endOffset  = end+bufStart; <br>
		 * 其中 bufStart 为最近一个{@link AudioDetector#reset()}调用后的字节偏移。
		 * 因为 end 值是不受状态返回延迟由引擎计算到的偏移，所以不需要考虑状态返回延迟
		 * 而造成的偏移。
		 * </p>
		 * <p>
		 * 		仅在{@link #status}为1或2时出现。
		 * </p>
		 * 
		 * <p>值类型：int</p>
		 * <p>值域：[0, +∞)</p>
		 */
		public int end = 0;
		
		/**
		 * <h4>错误信息</h4>
		 * <p>
		 * 		当些键值存在，且不为0时，表示出现错误。参考{@link com.iflytek.cloud.ErrorCode}，
		 * 以及{@link Error}。
		 * </p>
		 * 
		 * <p>值类型：int</p>
		 * <p>值域：(-∞, +∞)</p>
		 */
		public int error = 0;
		
		/**
		 * <h4>音频得分</h4>
		 * <p>
		 * 		音频参考得分，默认为1，仅在{@link AudioDetector#VAD_ENGINE}  为
		 * {@link AudioDetector#TYPE_META} 时生效。
		 * </p>
		 * 
		 * <p>值类型： float </p>
		 * <p>值域：[0,1]</p>
		 */
		public float confidence = 1;
		
		public static final int STATUS_OK = 0;
		public static final int STATUS_START = 1;
		public static final int STATUS_EOS = 2;
		public static final int STATUS_BOS = 3;
		public static final int STATUS_TIMEOUT = 4;
		
		public static final int SUB_OK = 0;
		public static final int SUB_START = 1;
		public static final int SUB_END = 2;
		public static final int SUB_STARTEND = 3;
		
	}//end of class Keys
	
	/**
	 * <h3>检测返回的错误类</h3>
	 * <p>
	 * 		定义了常见的几个错误，可能还有不在此类列出的错误码。
	 * </p>
	 */
	public static class Error{
		/**
		 * <h4>正常</h4>
		 */
		public static final int OK = 0;
		
		/**
		 * <h4>无效的参数</h4>
		 */
		public static final int INVALID_PARAM = 1;
		
		/**
		 * <h4>调用错误</h4>
		 */
		public static final int INVALID_INVOKE = 2;
		
		/**
		 * <h4>缓冲区不足</h4>
		 */
		public static final int BUFFER_NOT_ENOUGH = 3;
		
		/**
		 * <h4>缓冲区满</h4>
		 */
		public static final int BUFFER_FULL = 4;
		
		/**
		 * <h4>内存溢出</h4>
		 */
		public static final int OUT_OF_MEMORY = 12;
		
		/**
		 * <h4>没有结果</h4>
		 */
		public static final int RESULT_NONE = 13;
		
		/**
		 * <h4>不支持的参数</h4>
		 */
		public static final int PARAM_NOT_SUPPORT = 14;
	}
	
	/**
	 * <h4>是否尽快输出</h4>
	 * <p>
	 * 		仅在{@link #VAD_ENGINE 引擎类型}为{@link #TYPE_FIXFRONT}时起作用。
	 * 		设置为尽快输出时，将不检测前端点；否则将检测前端点。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：1</p>
	 * <p>值范围：{0，1}</p>
	 */
	public static final String EARLY_START = "early_start";
	
	/**
	 * <h4>分句超时</h4>
	 * <p>
	 * 		仅在{@link #VAD_ENGINE 引擎类型}为{@link #TYPE_FIXFRONT}时起作用。
	 * 每个分句最长的时间值（单位为毫秒）。当超过设置的值，还没找到分句结
	 * 束时（见{@link DetectorResult#sub}），SDK会返回分句超时的状态，见
	 * {@link DetectorResult#status}。在实现通过后端点来切分音频的需求时，建议
	 * 把此值设置为大于要切分的时长上限值，避免检测时未到达切分时长的上限值，
	 * 但超过分句超时值，仍未找到分句结束，而返回分句超时的状态。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：20000</p>
	 * <p>值范围：{0，+∞}</p>
	 */
	public static final String SUB_TIMEOUT = "sub_timeout";
	
	/**
	 * <h4>引擎类型</h4>
	 * <p>
	 * 		创建实例（{@link #createDetector(Context, String)}）时，
	 * 指定不同的引擎类型，以使用不同引擎。
	 * </p>
	 * <p>
	 * 		注意：如果当前已经有实例，要创建不同引擎类型的实例时，
	 * 必须先把原来的实例销毁（{@link #destroy()}），否则，创建
	 * 实例（{@link #createDetector(Context, String)}）时将直接
	 * 返回原来已有的实例，而不是新引擎的实例。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：{@link #DEF_ENGINE_TYPE}}</p>
	 * <p>值范围：{ {@link #TYPE_FIXFRONT}，{@link #TYPE_META} }</p>
	 */
	public static final String VAD_ENGINE = "vad_engine";
	
	/**
	 * <h4>资源路径</h4>
	 * <p>
	 * 		当{@link #VAD_ENGINE 引擎类型}为{@link #TYPE_META}时,
	 * 需要指定资源目录方可正常使用。资源路径值需使用 
	 * {@link ResourceUtil#generateResourcePath} 生成的资源路径格式。
	 * </p>
	 * 
	 * <p>是否必须设置：视引擎类型而定</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的资源目录字符串值</p>
	 */
	public static final String RES_PATH = "vad_res_path";
	
	/**
	 * <h4>灵敏度</h4>
	 * <p>
	 * 		仅在{@link #VAD_ENGINE 引擎类型}为{@link #TYPE_META}时起作用。
	 * 		对语音进行更严格的判断，值越高拒识越高，把噪声拒识的同时，也可能把有
	 * 效语音也拒识。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0.6</p>
	 * <p>值范围：[0, 1]</p>
	 */
	public static final String THRESHOLD = "threshold";
	
	/**
	 * <h4>额外的初始化参数</h4>
	 * <p>
	 * 		暂不支持。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：null 或 引擎支持的额外初始化参数值</p>
	 */
	protected static final String EXTRA = "extra";
	
	/**
	 * <h4>常量值：是否开启短音频过滤</h4>
	 * <p>
	 * 		当开启短音频过滤时，即使没有达到后端点指定的静音值，若引擎计算
	 * 认为当前后面的音频已是静音，就会直接返回后端点状态。
	 * 仅在{@link #VAD_ENGINE 引擎类型}为{@link #TYPE_META}时起作用。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{0, 1}</p>
	 * 
	 * @see #VAD_ENGINE
	 * @see #TYPE_META
	 */
	public final static String REDUCE_FLOW = "vad_reduce_flow";
	
	/**
	 * <h4>重设句子</h4>
	 * <p>
	 * 		当后面检测的音频，与前面的音频，没有上下文关系时（如，当前已结束音频文件A，
	 * 将开始检测音频文件B，而两个文件音频内容是独立的），通过设置此参数，使引擎清空
	 * 检测缓存，以避免影响后续音频返回的 {@link DetectorResult#start} 和 {@link DetectorResult#end}
	 * 的值。仅在{@link #VAD_ENGINE 引擎类型}为{@link #TYPE_META}时起作用。
	 * </p>
	 * <p>
	 * 		此参数仅在当次设置时生效。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：无</p>
	 * <p>值范围：{ 1 }</p>
	 * 
	 * @see #VAD_ENGINE
	 * @see #TYPE_META
	 */
	public final static String RESET_SENTENCE = "reset_sentence";
	
	/**
	 * <h4>常量值：最大缓存大小</h4>
	 * <p>
	 * 		检测音频时，应用层一次传入音频的最大字节大小。
	 * 当一次传入最大音频字节大于此值时，可能检测失败。
	 * 同时，检测后返回的有效音频可能会小于此值。
	 * </p>
	 * @see #detect
	 */
	public final static int MAX_BUF_LEN = 1024 * 32;
	
	/**
	 * <h4>常量值：能量引擎类型</h4>
	 * 
	 * @see #VAD_ENGINE
	 * @see #TYPE_META
	 */
	public final static String TYPE_FIXFRONT = "fixfront";
	
	/**
	 * <h4>常量值：模型引擎类型</h4>
	 * 
	 * @see #VAD_ENGINE
	 * @see #TYPE_FIXFRONT
	 */
	public final static String TYPE_META = "meta";
	
	/**
	 * <h4>默认引擎类型</h4>
	 */
	public final static String DEF_ENGINE_TYPE = TYPE_FIXFRONT;
	
	/**
	 * <h4>默认采样率</h4>
	 */
	public static final int DEF_RATE = 16000;
	
	/**
	 * <h4>默认前端点值</h4>
	 */
	public static final int DEF_BOS = 2000;
	
	/**
	 * <h4>默认后端点值</h4>
	 */
	public static final int DEF_EOS = 700;
	
	protected static AudioDetector sInstance = null;
	protected static Object sSyncObj = new Object();
	
	protected static final int VOLUME_MIN = 0;
	protected static final int VOLUME_MAX = 30;
	
	protected static final float DEF_CONFIDENCE = 1.0f;
	
	protected AudioDetector( Context context, String param ){}
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getDetector}获取已创建的单例对象。
	 * </p>
	 * 
	 * @param context 应用上下文
	 * @param param 初始化参数 包括： 
	 * <br>{@link com.iflytek.cloud.SpeechConstant#SAMPLE_RATE}：采样率；
	 * <br>{@link #VAD_ENGINE}：引擎类型；
	 * <br>{@link #RES_PATH}：资源目录；
	 * </p>
	 * @return 检测对象
	 * @see #destroy
	 * @see #getDetector
	 */
	public static AudioDetector createDetector( Context context, String param ){
		DebugLog.LogD( "createDetector enter, context: "+context+", param: "+param );
		
		synchronized( sSyncObj ){
			if( null == sInstance ){
				sInstance = detectorFactory( context, param );
			}//end of if sInstance is nulls
		}//end of synchronized
		
		DebugLog.LogD( "createDetector leave" );
		return sInstance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createDetector}创建单例对象。
	 * </p>
	 * 
	 * @return 检测对象
	 * 
	 * @see #createDetector
	 * @see #destroy
	 */
	public static AudioDetector getDetector(){
		synchronized( sSyncObj ){
			//synchronize with createAudioDetector
			DebugLog.LogD( "getDetector enter" );
		}
		
		return sInstance;
	}
	
	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createDetector}创建的单例对象。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createDetector}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createDetector
	 */
	abstract public boolean destroy();
	
	/**
	 * <h4>重置</h4>
	 * <p>
	 * 		重新开始新的端点检测。完成一次检测后，在下一次检音频前调用。
	 * </p>
	 * 
	 * @see #detect
	 */
	abstract public void reset();
	
	/**
	 * <h4>检测音频</h4>
	 * <p>
	 * 		传入待检测音频数据，同步返回检测结果。当传入的音频数据，是当前检测的
	 * 最后一段音频时，设置传入的数据为最后一段音频。当完成一次音频检测，进行
	 * 新的音频的第一段音频检测前，调用{@link #reset}重置引擎。
	 * </p>
	 * <p>
	 * 		每次传入的数据长度，应为1帧音频大小的整数倍，如音频为16位、单声道的
	 * Windows PCM音频，则数据长度应为 16/8=2字节的整数倍。当需要每小段音频
	 * 的较准确音量值时（如动态在UI中模拟音量变化时），建议每段写入的数据长度
	 * 相对的小一些，如为0.5秒的音频对应的字节数。
	 * </p>
	 * <p>
	 * 		此函数将把音频检测后，同步通过返回值返回，将阻塞当前线程。
	 * </p>
	 * @param buffer 待检测音频缓存区
	 * @param offset 有效音频开始位置编移
	 * @param length 有效音频数据长度。单次写入的最大长度限制，见{@link #MAX_BUF_LEN}，
	 * 			且应同时小于 {@link SpeechConstant#VAD_EOS} 值对应的音频字节长度。建议每次写入音频长度为 1600 左右。
	 * @param isLast 是否是最后一段音频
	 * @return 检测结果 参考{@link DetectorResult}
	 * 
	 * @see DetectorResult
	 */
	abstract public DetectorResult detect( byte[] buffer, int offset, int length, boolean isLast );
	
	/**
	 * <h4>设置参数</h4>
	 * <p>
	 * 		在检测前，可设置检测时的参数，包括：
	 * <br>{@link com.iflytek.cloud.SpeechConstant#VAD_BOS}：前端点长度，默认 2000 ms，值域[2000, 10000]；
	 * <br>{@link com.iflytek.cloud.SpeechConstant#VAD_EOS}：后端点长度，默认值 700 ms，值域[500, {@link Integer#MAX_VALUE}]；
	 * <br>{@link #SUB_TIMEOUT}：分句超时；
	 * <br>{@link #EARLY_START} : 是否尽快输出；
	 * <br>{@link #THRESHOLD}：灵敏度;
	 * <br>{@link com.iflytek.cloud.SpeechConstant#KEY_SPEECH_TIMEOUT}：音频超时，默认 -1（无穷）；
	 * <br>{@link #REDUCE_FLOW}：短音频过滤；
	 * 	<br>	目前编码时仅支持精度为16位，采样率为16000或8000，单声道，字节顺序为
	 *  Little-Endian的Windows PCM音频。
	 *  </p>
	 *  
	 * @param key 参数名
	 * @param value 参数值
	 */
	abstract public void setParameter( String key, String value );

	/*
	 * VAD工厂
	 * 通过在 param 中，指定ENGINE_TYPE，返回不同的对象。
	 */
	private static AudioDetector detectorFactory( Context context, String param ){
		AudioDetector detector = null;
		
		HashParam params = new HashParam();
		params.initWithParam( param );
		
		final String libName = params.getString( SpeechConstant.LIB_NAME );
		if( TextUtils.isEmpty(libName) || loadLibrary( libName ) ){
			final String engineType = params.getString( VAD_ENGINE, DEF_ENGINE_TYPE );
			if( TYPE_FIXFRONT.equalsIgnoreCase(engineType) ){
				detector = new AudioDetectorImplFixfrontVad( context, param );
			}else if( TYPE_META.equalsIgnoreCase(engineType) ){
				detector = new AudioDetectorImplMetaVad( context, param );
			}else{
				DebugLog.LogE( "detector factory unmatched engine type: "+engineType );
			}//end of if-else engineType
		}else{
			DebugLog.LogE( "detector factory load library failed: "+libName );
		}
		//end of if 
		
		return detector;
	}
	
	/*
	 * 加载库
	 */
	private static  boolean loadLibrary( String loadName ){
		boolean ret = true;
		if( !TextUtils.isEmpty(loadName) ){
			try{
				System.loadLibrary( loadName );
			}catch(Throwable e){
				ret = false;
				DebugLog.LogE( "Load library failed." );
				e.printStackTrace();
			}//end of try-catch
		}//end of if loadName is empty
		
		return ret;
	}//end of function loadLibrary
}
